//-------------------------------------------------------------------------------------------------------
//      Copyright (c) 2005, Be Nanashi-San 
// 
//      Permission is hereby granted, free of charge, to any person obtaining a 
//      copy of this software and associated documentation files (the "Software"), 
//      to deal in the Software without restriction, including without limitation 
//      the rights to use, copy, modify, merge, publish, distribute, sublicense, 
//      and/or sell copies of the Software, and to permit persons to whom the 
//      Software is furnished to do so, subject to the following conditions: 
// 
//      The above copyright notice and this permission notice shall be included in 
//      all copies or substantial portions of the Software. 
// 
//      THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
//      IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
//      FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
//      AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//      LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//      FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
//      DEALINGS IN THE SOFTWARE. 
// 
//      File Name:   	Base2chTextView.cpp
//      Author:      	Be Nanashi-San@598 in "BeOS - Zeta R4" Thread 
//						(http://pc8.2ch.net/test/read.cgi/os/1120676818/598)
//		Description:	互換Base2chTextView.
//						VIEW_LIMITERで表示するレス数を制限できる(待ち時間が我慢ならないようなら)
//						PostMessage()使いまくり。Zetaでは-D_ZETA_USING_DEPRECATED_API_が必要
//						DisplayThreadViewからとMStickerかも使われ、条件が多少違うので煩わしくなっている。
//-------------------------------------------------------------------------------------------------------
#include <Application.h>		// be_app
#include <Beep.h>				// beep
#include <Message.h>
#include <Messenger.h>
#include <Roster.h>				// be_roster
#include <View.h>
#include <Window.h>
#include <ctype.h>
#include <kernel/OS.h>			// for thread
#include <stdio.h>
#include <stdlib.h>				// atoi

#include "messages.h"

#include "Base2chTextView.h"
#include "MonaList.h"
#include "MUtils.h"

////////////////////////////////////////////////////////////////////////////////
// quote_strings
// a href = "...." の中身をコピー
// 20051229:コピー削除。アドレスとサイズを返す。
////////////////////////////////////////////////////////////////////////////////
static const char*
quote_strings(const char *data, uint32 length, int32 &size)
{
	const char *text = NULL;
	uint32 i;
	
	for (i = 0; i < length - 1; i++){
	 	if (data[i] == '"') {i++; text = data + i; break;}
	}
	size = 0;
	for (; i < length; i++, size++){
		if (data[i] == '"') break;
	}

	return text;
}
////////////////////////////////////////////////////////////////////////////////
// Unicode2UTF8
// from bemail
// 2005.12.14:Monalist.cppより移動。
////////////////////////////////////////////////////////////////////////////////
static int32
Unicode2UTF8(int32 c, char **out)
{
	char *s = *out;
	int32 ret = 0;
//	ASSERT(c < 0x200000);

	if (c < 0x80){
		*(s++) = c;
		ret = 1;
	}else if (c < 0x800)
	{
		*(s++) = 0xc0 | (c >> 6);
		*(s++) = 0x80 | (c & 0x3f);
		ret = 2;
	}
	else if (c < 0x10000)
	{
		*(s++) = 0xe0 | (c >> 12);
		*(s++) = 0x80 | ((c >> 6) & 0x3f);
		*(s++) = 0x80 | (c & 0x3f);
		ret = 3;
	}
	else if (c < 0x200000)
	{
		*(s++) = 0xf0 | (c >> 18);
		*(s++) = 0x80 | ((c >> 12) & 0x3f);
		*(s++) = 0x80 | ((c >> 6) & 0x3f);
		*(s++) = 0x80 | (c & 0x3f);
		ret = 4;
	}
	*out = s;
	return ret;
}
////////////////////////////////////////////////////////////////////////////////
// htmlspecialchar2utf8
// html特殊文字の変換。
// 消費した長さを返し、utf8charには変換した文字を返す。
// 2005.12.14:Monalist.cppより移動。
////////////////////////////////////////////////////////////////////////////////
static int
htmlspecialchar2utf8(const char *s, char *utf8char, int32 &outlen)
{
	if (s[0] == '<') {
		if (s[1] == 'b' && s[2] == '>' ){
			*utf8char = 0;
			outlen = 0;
			return 3;
		}
		
		if (s[1] == '/' && s[2] == 'b' && s[3] == '>' ){
			*utf8char = 0;
			outlen = 0;
			return 4;
		}
	
		int32 i = 1;
		for (const char *br = "br>"; br[0]; br++) {
			while (s[i] == ' ') i++;
			if (br[0] != s[i]){
				outlen = 0;
				return 0;
			}
			i++;
		}
		
	 	*utf8char = '\n';
		outlen = 1;
		return i;
		
	} else if (*s == '&') {

		if(*(s + 1) == '#'){
			char buf[256];
			int32 i = 2; char *p = buf;
			for(; isdigit(s[i]) ; i++) *p++ = s[i];
			*p = 0;
			int32 c = atol(buf);
			outlen = Unicode2UTF8(c, &utf8char);
			
			return i + 1;
		}

		struct convert_info{
			int32 used_length;
			char utf8char;
			char *htmlchar;
		};
		
		const convert_info htmlchars[] = {
			{4, '>', "&gt;"},
			{4, '<', "&lt;"},
			{6, ' ', "&nbsp;"},
			{6, '\"', "&quot;"},
			{5, '&', "&amp;" }
			};

		for (uint32 i = 0; i < sizeof(htmlchars)/sizeof(convert_info); i++) {
			int length = htmlchars[i].used_length;
			if (strncmp(s, htmlchars[i].htmlchar, length) == 0){
				*utf8char =  htmlchars[i].utf8char;
				outlen = 1;
				return length;
			}
		}
	}
	*utf8char = 0;
	return 0;
}

////////////////////////////////////////////////////////////////////////////////
// Base2chTextView
//
//
////////////////////////////////////////////////////////////////////////////////
Base2chTextView::Base2chTextView(BRect frame, const char *name, BRect textRect, uint32 resizingMode, uint32 flags)
	: BTextView(frame, name, textRect, resizingMode, flags|B_WILL_DRAW|B_FRAME_EVENTS|B_FULL_UPDATE_ON_RESIZE)
	, fAction(this)
	, fCountTimer(5)

{
	BTextView::SetStylable(true);
}
////////////////////////////////////////////////////////////////////////////////
// `Base2chTextView
//
//
////////////////////////////////////////////////////////////////////////////////
Base2chTextView::~Base2chTextView()
{

}
////////////////////////////////////////////////////////////////////////////////
// AttachedToWindow
//
//
////////////////////////////////////////////////////////////////////////////////
void
Base2chTextView::AttachedToWindow()
{
	BTextView::MakeEditable(false);
	BTextView::MakeSelectable(true);
}
////////////////////////////////////////////////////////////////////////////////
// Insert
//
//
////////////////////////////////////////////////////////////////////////////////
//Casheに変更。
////////////////////////////////////////////////////////////////////////////////
// MessageReceived
//
//
////////////////////////////////////////////////////////////////////////////////
void
Base2chTextView::MessageReceived(BMessage *msg)
{
	BTextView::MessageReceived(msg);
}
////////////////////////////////////////////////////////////////////////////////
// MouseDown
//
//
////////////////////////////////////////////////////////////////////////////////
void
Base2chTextView::MouseDown(BPoint where)
{
	BTextView::MouseDown(where);
	action_info* action = fAction.ItemTo(OffsetAt(where));
	fCursorOffset = (action) ? action->offset : -1;
	fMousePoint = where;
}
////////////////////////////////////////////////////////////////////////////////
// MouseUp
//
//
////////////////////////////////////////////////////////////////////////////////
void
Base2chTextView::MouseUp(BPoint where)
{
	action_info* action = fAction.ItemTo(OffsetAt(where));
	if (action) {
		fCursorOffset = action->offset;
		fCountTimer = 5;
		this->Invoke();
	}else{
		fCursorOffset = -1;
	}
	
	BRect spot = BRect(fMousePoint, fMousePoint).InsetByCopy(-2, -2);
	if (spot.Contains(where))	/* Click(ほぼ同じ場所でDown、Up)か */
		this->SelectionInvoke();

	BTextView::MouseUp(where);
}
////////////////////////////////////////////////////////////////////////////////
// MouseMoved
//
//
////////////////////////////////////////////////////////////////////////////////
void
Base2chTextView::MouseMoved(BPoint where, uint32 transit, const BMessage *msg)
{
	BTextView::MouseMoved(where, transit, msg);

	if (transit == B_INSIDE_VIEW) {
		int32 offset = fCursorOffset;
		action_info* action = fAction.ItemTo(OffsetAt(where));
		fCursorOffset = (action) ? action->offset : -1;
		if (where != fStatic_where) {
		/* B_INSIDE_VIEWはMouseMoveしていなくても呼ばれている?ので */
			fCountTimer = 5;					/* reset */
			fStatic_where = where;
		}
	
		if (fCursorOffset != offset) PostStatus(action ? action->name : "");

	}
}
////////////////////////////////////////////////////////////////////////////////
// Pulse
//
//
////////////////////////////////////////////////////////////////////////////////
void
Base2chTextView::Pulse()
{
	if (Window()->IsActive()) {
		BTextView::Pulse();	/* 必要か疑問 */
		if (--fCountTimer == 0) {
			action_info* action = fAction.ItemTo(fCursorOffset);
			if (action && strncmp(action->name, "../", 3) == 0){
				this->Invoke();
			}else{
				this->SelectionInvoke();
			}
			fCountTimer = 5;
		}
	}
}
////////////////////////////////////////////////////////////////////////////////
// Launcher
//
//
////////////////////////////////////////////////////////////////////////////////
void
Base2chTextView::Launcher(const char *type, const char *param)
{
	char *argv = strdup(param);
	be_roster->Launch(type, 1, &argv);
	free(argv);
}
////////////////////////////////////////////////////////////////////////////////
// PostStatus
//
// 20051120:classに移動。WindowAt()からBiew::Window()へ変更。
////////////////////////////////////////////////////////////////////////////////
void
Base2chTextView::PostStatus(const char *title)
{
}

////////////////////////////////////////////////////////////////////////////////
// Invoke
//
//
////////////////////////////////////////////////////////////////////////////////
status_t
Base2chTextView::Invoke()
{
	const action_info* action = fAction.Current();
	if (action) {
		if (strncmp(action->name, "http://", 7) == 0){
			Launcher(B_URL_HTTP, action->name);
		}else if (strncmp(action->name, "../", 3) == 0) {
			this->InvokeThread(atoi(strrchr(action->name, '/') + 1));
		}
	}
	return B_OK;
}
////////////////////////////////////////////////////////////////////////////////
// InvokeThread
//
//
////////////////////////////////////////////////////////////////////////////////
void
Base2chTextView::InvokeThread(int32 thread_no)
{
#if DEBUG
	printf("InvokeThread(%ld)\n", thread_no);
#endif
	BMessage msg(kMSG_BORN_BABY);
	msg.AddInt32("thread_no", thread_no);
	BMessenger(be_app->WindowAt(0)->FindView("2ch:TextView")).SendMessage(&msg);
}
////////////////////////////////////////////////////////////////////////////////
// is_number
//
// 20060408：BTextView::ByteAt()の呼び出しに修正。
////////////////////////////////////////////////////////////////////////////////
int32
Base2chTextView::is_number(int32 offset)
{
	int32 length = offset;
	while (isdigit(BTextView::ByteAt(length))) length++;
	if (length > offset) return length - offset;

//	20051230:全角数字
	length = offset;
	while (BTextView::ByteAt(length) == 0xef){
		if (BTextView::ByteAt(length + 1) != 0xbc )	break;
		if(BTextView::ByteAt(length + 2) < 0x90 || BTextView::ByteAt(length + 2) > 0x99) break;
	 	length += 3;
	}

	return length - offset;

}
////////////////////////////////////////////////////////////////////////////////
// SelectionInvoke
// Selectionされた範囲が数字ならレス番号として起動する */
//
////////////////////////////////////////////////////////////////////////////////
void
Base2chTextView::SelectionInvoke()
{
	int32 start, finish;
	BTextView::GetSelection(&start, &finish);
	if (start < finish) {
		int32 size = is_number(start );	/* Selectionされたものが数字か */
		if (size == (finish - start) && size  < 5) {
			char buf[8];
			BTextView::GetText(start, size, buf);// 20060518syuusei
			buf[size] = 0;
			int i = atoi(buf);
			this->InvokeThread(i);
		}
	}
}
////////////////////////////////////////////////////////////////////////////////
// FrameResized
// 20060507:CalcBounds廃止
//
////////////////////////////////////////////////////////////////////////////////
void
Base2chTextView::FrameResized(float width, float height)
{
	this->SetTextRect(BRect(3, 3, width - 6, height - 6));
}
////////////////////////////////////////////////////////////////////////////////
// ScrollTo
//
//
////////////////////////////////////////////////////////////////////////////////
void
Base2chTextView::ScrollTo(BPoint point)
{
	BTextView::ScrollTo(point);
}
////////////////////////////////////////////////////////////////////////////////
// HtmlParser
//
//
////////////////////////////////////////////////////////////////////////////////
void
Base2chTextView::HtmlParser(const char *text)
{
	const char *action = NULL;
	int32 actionsize = 0;
	
	int32 offset = 0, i;
	for (i = 0; text[i]; i++) {
		//2005.12.14:779氏@BeOS - Zeta R4より。
		char outchar[5]; 
		int32 outlen; 
		int32 changechars = htmlspecialchar2utf8(text + i, outchar, outlen); /* 実際の長さ以上に参照している */ 
		if (changechars) { 
			if (i - offset) Insert(text + offset, i - offset, MStyle::PLAIN()); 
			Insert(outchar, outlen, MStyle::PLAIN()); 
			i += changechars; 
			offset = i--; 
		}else if (text[i] == '<') {
			if (text[i + 1] == 'a') { 				// start tag
				if (i - offset) this->Insert(text + offset, i - offset, MStyle::PLAIN());
				for (int32 j = i + 2; text[j]; j++) {
					if (text[j] == '>') {
						action = quote_strings(text + i + 2, j - 1 - (i + 2), actionsize);
						i = j;
						offset = i + 1;
						break;
					}
				}
			}else if (text[i + 1] == '/') {		// end tag
				this->SetInvoke(action, actionsize, i - offset);
				this->Insert(text + offset, i - offset, MStyle::LINK());
				action = NULL;										// タグを閉じるしるしとして
				for (int32 j = i + 2; text[j]; j++) {
					if (text[j] == '>') {
						i = j;
						offset = i + 1;
						break;
					}
				}
			}
		}else if (action == NULL && text[i] == '>' ) {	// >numberはアンカーされないので
			while (isspace(text[i+1])) i++;		// space cut
			int32 length = is_number(i + 1);
			if (length) {
				this->Insert(text + offset, i + 1 - offset, MStyle::PLAIN());
				//20051107
				char url[256];// = "../99999";
				strcpy(url, "../");
				memcpy(url + 3, text + i + 1, length);
				url[3 + length] = '\0';
				this->SetInvoke(url, length);
				this->Insert(text + i + 1, length, MStyle::LINK());
				offset = i + 1 + length;
			}
		}else if (text[i] == 'h' && strncmp(text + i, "http://", 7) == 0) {
			if (i - offset) this->Insert(text + offset, i - offset, MStyle::PLAIN());
			offset = i;
			i += 7;
			while (text[i]) {
				if (isspace(text[i]) || uint8(text[i]) & 0x80) break;
				i++;
			}
			this->SetInvoke(text + offset, i - offset, i - offset);
			this->Insert(text + offset, i - offset, MStyle::LINK());
			offset = i;
		}
	}
	if (i - offset) Insert(text + offset, i - offset, MStyle::PLAIN());
}
////////////////////////////////////////////////////////////////////////////////
// SetInvoke
//	現在のテキスト位置(TextLength())からlengthまでをaction範囲とする。
//
////////////////////////////////////////////////////////////////////////////////
void 
Base2chTextView::SetInvoke(const char *name, int32 length)
{
	//20051123
	if (name) this->SetInvoke(name, strlen(name), length);
}

////////////////////////////////////////////////////////////////////////////////
// SetInvoke
//	現在のテキスト位置(TextLength())からlengthまでをaction範囲とし
//	action名としてnameからsize分が使われる
////////////////////////////////////////////////////////////////////////////////
void
Base2chTextView::SetInvoke(const char *name, int32 size, int32 length)
{
	//20060528
	if(!name) return;
	
	action_info *action = new action_info;
	action->offset = this->TextLength();
	action->length = (length < 0) ? strlen(name) : length;
	char *buf = static_cast<char *>(malloc(size + 1));
	strncpy(buf, name, size);
	buf[size] = '\0';
	action->name = buf;
	action->size = size;
	fAction.AddAction(action);
}
////////////////////////////////////////////////////////////////////////////////
// PrintToStream
//
//
////////////////////////////////////////////////////////////////////////////////
void
Base2chTextView::PrintToStream(int32 flags)
{
	const char *text = Text();
	if (flags == 0) {
		int32 count = fThread.size();
		int32 left	= 0;
		printf("<Thread(%ld/%ld)\n", TextLength(), count);
		for (int32 i= 0; i < count; i++) {
			int32 offset = fThread[i];
			BString str(text + left, offset - left);
			printf("%5ld:(%s)\n", left, str.String());
			left = offset;
		}
		printf("Thread>\n");
	}
}
