//------------------------------------------------------------------------------ 
//      Copyright (c) 2004, Be Nanashi-San 
// 
//      Permission is hereby granted, free of charge, to any person obtaining a 
//      copy of this software and associated documentation files (the "Software"), 
//      to deal in the Software without restriction, including without limitation 
//      the rights to use, copy, modify, merge, publish, distribute, sublicense, 
//      and/or sell copies of the Software, and to permit persons to whom the 
//      Software is furnished to do so, subject to the following conditions: 
// 
//      The above copyright notice and this permission notice shall be included in 
//      all copies or substantial portions of the Software. 
// 
//      THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
//      IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
//      FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
//      AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//      LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//      FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
//      DEALINGS IN THE SOFTWARE. 
// 
//      File Name:   BoardListView.cpp
//      Author:      Be Nanashi-San 
//      Description: BBS Board list display class. 
//------------------------------------------------------------------------------

#include <Beep.h>
//#define DEBUG 1
#include <Debug.h>
#include <Directory.h>
#include <Entry.h>
#include <File.h>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include <MenuField.h>
#include <Mime.h>
#include <Window.h>
#include <ScrollView.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "2chPositive.h"
#include "BoardItem.h"
#include "BoardListView.h"
#include "BoardOutLineListView.h"
#include "Bookmark2ch.h"
#include "NetUtil.h"
#include "ThreadListView.h"
#include "Util.h"
#include "messages.h"
#include "setting.h"
#include "BBSHandler.h"

static const unsigned int kNAMEBUFSIZ = 256;
static const unsigned int kBBSCHANGE = 'kBBS';

#define BRECT(x,y,w,h) BRect((x), (y), (x+w-1), (y+h-1))


////////////////////////////////////////////////////////////////////////////////
// BoardListView
//
//
////////////////////////////////////////////////////////////////////////////////
BoardListView::BoardListView(BRect rect)
: BView(rect, "BoardList", B_SINGLE_SELECTION_LIST, B_FOLLOW_ALL)
,fStopLoading(false)
,fThread(0)
{
	fStopSem = create_sem(1, "BoardListReader_sem");
	this->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
}
////////////////////////////////////////////////////////////////////////////////
// ~BoardListView
//
//
////////////////////////////////////////////////////////////////////////////////
BoardListView::~BoardListView()
{
	delete_sem(fStopSem);
}
////////////////////////////////////////////////////////////////////////////////
// AttachedToWindow
//
//
////////////////////////////////////////////////////////////////////////////////
void
BoardListView::AttachedToWindow() 
{ 
	BPopUpMenu *aMenu = new BPopUpMenu("");
	BMenuField *aPopMenu = new BMenuField(BRect(1,1,250,19), NULL, NULL, aMenu);
	static_cast<a2chViewer *>(be_app)->MakeReaderList(aMenu);

	aMenu->SetTargetForItems(this);
	aPopMenu->SetDivider(0.0f);
	this->AddChild(aPopMenu);

	fCureader = static_cast<a2chViewer *>(be_app)->CurReader();

	BRect rect = this->Bounds();
	rect.right -= (B_V_SCROLL_BAR_WIDTH + 3);
	rect.left += 3;
	rect.top += 30;
	rect.bottom -= 3;
	fOutLineView = new BoardOutLineListView(rect);
	BScrollView *aScrollView = new BScrollView("BoardListScrollView",fOutLineView , B_FOLLOW_ALL_SIDES, 0, false, true);

	this->AddChild(aScrollView);

	/*status_t sts = */this->SetBBS();
	//TODO:リスト作成に失敗したときはどうするか？（板一覧置き場の移動など）
	
	BView::AttachedToWindow();
} 
////////////////////////////////////////////////////////////////////////////////
// SetBBS
// 対象BBSを切り替える。現在は2ch(index=0)に固定
// forceRemoteが真の時、ローカルは無視してネットから読込む。
////////////////////////////////////////////////////////////////////////////////
status_t
BoardListView::SetBBS (bool forceNetLoad)
{
	status_t sts = B_ERROR;
	
	//Netからの読み込み強制なければローカルからの読み込み。
	if (!forceNetLoad){
		sts = fCureader->LoadFromLocal();
	}
	
	//Netからの読み込み強制またはローカルからの読み込み失敗。
	if(sts != B_OK){
		sts = fCureader->LoadFromInet();
	}

	this->StopDrawing();

	if(sts == B_OK){
		BoardListView::Reader *reader = new BoardListView::Reader(this, fStopSem);
		fThread = spawn_thread(Reader::Run, "BoardListReader", B_NORMAL_PRIORITY, reader);
		resume_thread(fThread);
	}

	return sts;
}
////////////////////////////////////////////////////////////////////////////////
// StopLoad
// 読込み用スレッドを停止。（MDRより）
//
////////////////////////////////////////////////////////////////////////////////
void
BoardListView::StopDrawing()
{
	Window()->Unlock();

	thread_info	info;
	if (fThread != 0 && get_thread_info(fThread, &info) == B_NO_ERROR) {
		fStopLoading = true;
		acquire_sem(fStopSem);
		int32 result;
		wait_for_thread(fThread, &result);
		fThread = 0;
		release_sem(fStopSem);
		fStopLoading = false;
	}

	Window()->Lock();
}
////////////////////////////////////////////////////////////////////////////////
// BoardListView::Reader::Reader
// dat読込みスレッド
// 20050627:datnameをポインタでなくコピーで保持するように修正。
////////////////////////////////////////////////////////////////////////////////
BoardListView::Reader::Reader(BoardListView *view, sem_id sem)
:
fView(view),
fStopSem(sem)
{
}
////////////////////////////////////////////////////////////////////////////////
// BoardListView::Reader::Run
//
//
////////////////////////////////////////////////////////////////////////////////
status_t
BoardListView::Reader::Run(void *_this)
{

	Reader *reader = static_cast<Reader *>(_this);
	
	if (!reader->Lock())
		return B_INTERRUPTED;

	BoardListView *view = reader->fView;

	if(view->fStopLoading) goto done;

	if(view->fOutLineView->IsEmpty() == false){
		for (;;){
			if (view->fStopLoading || !view->Window()->Lock()) goto done;
			BoardItem *item = static_cast<BoardItem *>(view->fOutLineView->RemoveItem(static_cast<int32>(0)));
			view->Window()->Unlock();
			if (item == NULL) break;
			delete item;
		}
		
		if (view->fStopLoading || !view->Window()->Lock()) goto done;
		view->fOutLineView->MakeEmpty(); //必要？
		view->Window()->Unlock();
	}

	if(view->fStopLoading) goto done;
	
	(view->fCureader)->DrawBoardList(view);


done:
	reader->Unlock();
	delete reader;
	return B_NO_ERROR;
}
////////////////////////////////////////////////////////////////////////////////
// ThreadListView::Reader::Unlock
//
//
////////////////////////////////////////////////////////////////////////////////
status_t
BoardListView::Reader::Unlock()
{
	return release_sem(fStopSem);
}
////////////////////////////////////////////////////////////////////////////////
// ThreadListView::Reader::Lock
//
//
////////////////////////////////////////////////////////////////////////////////
bool
BoardListView::Reader::Lock()
{
	if (acquire_sem_etc(fStopSem, 1, B_TIMEOUT, 0) != B_NO_ERROR)
		return false;

	return true;
}
////////////////////////////////////////////////////////////////////////////////
// BBSChanged
////////////////////////////////////////////////////////////////////////////////
void
BoardListView::BBSChanged (BMessage * message)
{
	void *item;
	message->FindPointer("source", &item);
	static_cast<a2chViewer *>(be_app)->SetCurReader(static_cast<BMenuItem *>(item)->Label());
	if(fCureader == static_cast<a2chViewer *>(be_app)->CurReader()) return;
	fCureader = static_cast<a2chViewer *>(be_app)->CurReader();
	this->SetBBS();
}

////////////////////////////////////////////////////////////////////////////////
// MessageReceived
//
//
////////////////////////////////////////////////////////////////////////////////
void
BoardListView::MessageReceived (BMessage * message)
{
	switch (message->what){
		// zeta 2 スレ@757のBe名無しさんより。
	
		case kMENU_BOARDLIST_REFREASH:
			//TODO: BBS切替に対応。
			this->SetBBS(true); // refresh from inet.
			break;
	
		case kMENU_BOARDLIST_EXPANDALL:
			fOutLineView->ExpandOrCollaspse(true);
			break;
	
		case kMENU_BOARDLIST_COLLAPSEALL:
			fOutLineView->ExpandOrCollaspse(false);
			break;

		case kMSG_FIND:
			if (fOutLineView->FindBoardName(message->FindString("findthis")) < 0)
				beep();
			break;

		case kBBSCHANGE:
			this->BBSChanged (message);
			break;

		default:
			PRINT(("BoardListView "));
			BView::MessageReceived (message);
			break;
	}
}
////////////////////////////////////////////////////////////////////////////////
// OutLineView
//  
//  
////////////////////////////////////////////////////////////////////////////////
BoardOutLineListView*
BoardListView::OutLineView()
{
	return fOutLineView;
}
