//-------------------------------------------------------------------------------------------------------
//      Copyright (c) 2005, Be Nanashi-San 
// 
//      Permission is hereby granted, free of charge, to any person obtaining a 
//      copy of this software and associated documentation files (the "Software"), 
//      to deal in the Software without restriction, including without limitation 
//      the rights to use, copy, modify, merge, publish, distribute, sublicense, 
//      and/or sell copies of the Software, and to permit persons to whom the 
//      Software is furnished to do so, subject to the following conditions: 
// 
//      The above copyright notice and this permission notice shall be included in 
//      all copies or substantial portions of the Software. 
// 
//      THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
//      IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
//      FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
//      AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//      LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//      FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
//      DEALINGS IN THE SOFTWARE. 
// 
//      File Name:   	K2chTextView.cpp
//      Author:      	Be Nanashi-San@598 in "BeOS - Zeta R4" Thread 
//						(http://pc8.2ch.net/test/read.cgi/os/1120676818/598)
//		Description:	互換K2chTextView.
//						VIEW_LIMITERで表示するレス数を制限できる(待ち時間が我慢ならないようなら)
//						PostMessage()使いまくり。Zetaでは-D_ZETA_USING_DEPRECATED_API_が必要
//						DisplayThreadViewからとMStickerかも使われ、条件が多少違うので煩わしくなっている。
//-------------------------------------------------------------------------------------------------------
#include <Application.h>		// be_app
#include <Beep.h>				// beep
#include <MenuItem.h>
#include <Message.h>
#include <Messenger.h>
#include <PopUpMenu.h>
#include <Clipboard.h>
#include <File.h>
#include <View.h>
#include <Debug.h>
#include <ctype.h>
#include <kernel/OS.h>			// for thread
#include <stdio.h>
#include <stdlib.h>				// atoi

#include "messages.h"
#include "datattr.h"

#include "K2chTextView.h"
#include "K2chTipTextView.h"
#include "MSticker.h"
#include "MonaList.h"
#include "MUtils.h"
#include "Util.h"


//一度に表示するスレ数。
static const int32 DISPLAYTHREADRANGE = 100;

// TODO:
//﻿・スレ読み込み時の挙動をどうするか？
//　既読の無い場合
//　　最新１００読み込み
//　　１－１００読み込み
//　既読のある場合
//　　既読から１００読み込み
//　　既読分は無視して、既読無い時と同じ挙動
//　　最新１００読み込み
//　　１－１００読み込み
//・スレReload時の挙動
//　全スレ表示モード、最新１００表示モード以外は無視する
//　（メニューを使用不可にする）
//　増分の処理
//　　既読の最後に追加。
//　　増分分から表示。
// TODO:表示外にあるレスアンカー表示。

////////////////////////////////////////////////////////////////////////////////
//  itoa
//
//
////////////////////////////////////////////////////////////////////////////////
static const char*
itoa(int32 n)
{
	static char digits[10];// = "123456789";
	digits[sizeof(digits) - 1] = 0;
	int32 d = sizeof(digits) - 2;
	while (0 < n) {
		digits[--d] = (n % 10) + '0';
		n /= 10;
	}
	return digits + d;
}
////////////////////////////////////////////////////////////////////////////////
// Clipboard_To
//
//
////////////////////////////////////////////////////////////////////////////////
static void
Clipboard_To(const char *data)
{
	if (be_clipboard->Lock()) {
		be_clipboard->Clear();
		BMessage *clipper = be_clipboard->Data();
		/* sizeとして文字列の終端(NULL文字)を入れてはならない
			入れるとBTextViewは続く一行を存在するが表示しないという奇妙な現象になる */
		clipper->AddData("text/plain", B_MIME_TYPE, data, strlen(data), false);
		be_clipboard->Commit();
		be_clipboard->Unlock();
	}
}
////////////////////////////////////////////////////////////////////////////////
// K2chTextView
//
//
////////////////////////////////////////////////////////////////////////////////
K2chTextView::K2chTextView(BRect frame, const char *name, BRect textRect, uint32 resizingMode, uint32 flags)
	: Base2chTextView(frame, name, textRect, resizingMode, flags|B_WILL_DRAW|B_FRAME_EVENTS|B_FULL_UPDATE_ON_RESIZE)
	,fReloaded(false)
	, fThread_id(0)
	, fCancel(false), fFile(NULL), fTipStiker(NULL)
{
		this->SetDispmode(DISP_PARTIAL_THREAD);
}
////////////////////////////////////////////////////////////////////////////////
// `K2chTextView
//
//
////////////////////////////////////////////////////////////////////////////////
K2chTextView::~K2chTextView()
{
	delete fFile;
	if (fTipStiker) fTipStiker->Close();
}
////////////////////////////////////////////////////////////////////////////////
// MessageReceived
//
//
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
		case kMSG_FIND:
			this->FindString(msg->FindString("findthis"));
			break;
		case kMSG_BORN_BABY:
			this->Clone(msg->FindInt32("thread_no"));
			break;
		case kMSG_THREAD_END:
			//this->UpdateThreadNo(fFile);
			break;
		case kMSG_GOOGLE: {
			GoogleURL google(msg->FindString("selected"));
			Launcher(B_URL_HTTP, google.String()); }
			break;
		case kMSG_COPY_LOCATION:
		case kMSG_COPY_THREAD:
			Clipboard_To(msg->FindString("text"));
			break;
		case kMENU_THREAD_GO_TOP100:
			this->Seek(0, SEEK_SET);
			this->SetDispmode(DISP_PARTIAL_THREAD);
			this->LoadText();
			break;
		case kMENU_THREAD_GO_LAST100:
			this->Seek(-DISPLAYTHREADRANGE, SEEK_END);
			this->SetDispmode(DISP_PARTIAL_THREAD);
			this->LoadText();
			break;
		case kMENU_THREAD_GO_PREV100:
			this->Seek(-DISPLAYTHREADRANGE, SEEK_CUR);
			this->SetDispmode(DISP_PARTIAL_THREAD);
			this->LoadText();
			break;
		case kMENU_THREAD_GO_NEXT100:
			this->Seek(DISPLAYTHREADRANGE, SEEK_CUR);
			this->SetDispmode(DISP_PARTIAL_THREAD);
			this->LoadText();
			break;
		case kMENU_THREAD_SHOW_All:
			this->Seek(0, SEEK_SET);
			this->SetDispmode(DISP_ALL_THREAD);
			this->LoadText();
			break;
		default:
			BTextView::MessageReceived(msg);
			break;
	}
}
////////////////////////////////////////////////////////////////////////////////
// MouseDown
//
//
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::MouseDown(BPoint where)
{
	uint32 buttons = Window()->CurrentMessage()->FindInt32("buttons");
	if (buttons == B_SECONDARY_MOUSE_BUTTON) {
		BPopUpMenu menu("2ch+ Poper");
		this->AddMenuItems(&menu, where);
		menu.SetTargetForItems(this);
		menu.Go(ConvertToScreen(where) - BPoint(5, 10), true);
	} else {
		Base2chTextView::MouseDown(where);
	}
}
////////////////////////////////////////////////////////////////////////////////
// WindowActivated
//
//
////////////////////////////////////////////////////////////////////////////////

void
K2chTextView::MakeFocus(bool focusState)
 {
	BTextView::MakeFocus(focusState);
	if (Parent()->Parent())
		(Parent()->Parent())->Invalidate();
 }
////////////////////////////////////////////////////////////////////////////////
// FindString
// 文字列(find)を現在のセレクション位置の次からスキャンを初めて見つかればSelect()し
// ScrollToSelection()で見えるようスクロールする。
//
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::FindString(const char *find)
{
	int32 start, finish;
	GetSelection(&start, &finish);

	const size_t nchars = strlen(find);
	const char *text = this->Text();
	int32 length = this->TextLength();

	for (int32 i = finish + 1; i < length; i++) {
		if (strncasecmp(find, text + i, nchars) == 0) {
			BTextView::Select(i, i + nchars);
			this->ScrollToSelection();
			return;
		}
	}

	for (int32 i = 0; i < start; i++) {
		if (strncasecmp(find, text + i, nchars) == 0) {
			Select(i, i + nchars);
			this->ScrollToSelection();
			return;
		}
	}
	beep();
}
////////////////////////////////////////////////////////////////////////////////
// PostStatus
//
// 20051120:classに移動。WindowAt()からBiew::Window()へ変更。
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::PostStatus(const char *title)
{
	BMessage msg(kMSG_STATUS_MESSAGE);
	msg.AddString("message", title);
	BMessenger(this->Window()).SendMessage(&msg);
}

////////////////////////////////////////////////////////////////////////////////
// ScrollToRes
//
//
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::ScrollToRes(int32 thread_no)
{
	int32 offset = fThread[thread_no - 1 - fCurPositon];
	if(offset){
		if (fThread_id) Window()->Lock();
		BTextView::ScrollTo(0, PointAt(offset).y);
		if (fThread_id) Window()->Unlock();
	}
}
////////////////////////////////////////////////////////////////////////////////
// ClearText
//
// 20051029:SetTextから名称変更。
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::ClearText()
{
	fThread.clear();
	fAction.Clear();

	BTextView::SetText("");
	PostStatus("");
}
////////////////////////////////////////////////////////////////////////////////
// LoadFile
//
//
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::LoadFile(BFile *file, bool reloaded)
{
	delete fFile;
	fFile = file;
	fDat.InitCheck();
	fDat.SetTo(file);
	this->UpdateThreadNo(file);
	fReloaded = reloaded;
	
//	if(!reloaded){
		this->Seek(0, SEEK_SET);
//	}
	this->LoadText();
}
////////////////////////////////////////////////////////////////////////////////
// StopLoad
//
//
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::StopLoad()
{
	fCancel = true;
	Window()->Unlock();
	PostStatus("Window Unlock. thread 終了待ち");
	status_t exit;
	wait_for_thread(fThread_id, &exit);	// for Cancel
	PostStatus("Window Locked");
	Window()->Lock();
}
////////////////////////////////////////////////////////////////////////////////
// LoadText
//
//
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::LoadText()
{
	if (fThread_id) {
		this->StopLoad();
	}

	this->ClearText();

	fThread_id = spawn_thread(InsertMonaRun, "InsertMona", B_NORMAL_PRIORITY, this);
	resume_thread(fThread_id);
	PostStatus("spawn_thread IntertMona.. .");
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
bool
K2chTextView::AcceptReload()
{
return true;
}
////////////////////////////////////////////////////////////////////////////////
// seek
//
//
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::Seek(int32 offset, uint32 seek_mode)
{
	if (seek_mode == SEEK_SET){
		fCurPositon = 0;
	}else if(seek_mode == SEEK_END){
		fCurPositon = fDat.CountItems();
	}
	
	fCurPositon += offset;
	if(fCurPositon < 0) fCurPositon = 0;
	if(fCurPositon > fDat.CountItems()) fCurPositon = fDat.CountItems();
}
////////////////////////////////////////////////////////////////////////////////
// seek
//
//
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::SetDispmode(disp_mode mode)
{
	fDispmode = mode;
}
////////////////////////////////////////////////////////////////////////////////
// seek
//
//
////////////////////////////////////////////////////////////////////////////////
const disp_mode
K2chTextView::Dispmode() const
{
	return fDispmode;
}

////////////////////////////////////////////////////////////////////////////////
// InsertMonaRun
//
//
////////////////////////////////////////////////////////////////////////////////
int32
K2chTextView::InsertMonaRun(void *data)
{
	K2chTextView* self = static_cast<K2chTextView*>(data);
	self->fCancel = false;
	self->InsertMona();
	if (!self->fCancel){
		BMessenger(self->Window()).SendMessage(kMSG_THREAD_END, self);
	}
	self->fThread_id = 0;
	return B_OK;
}
////////////////////////////////////////////////////////////////////////////////
// InsertMona
//
//
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::InsertMona()
{
	int32 start = fCurPositon;
	int32 last = (Dispmode() == DISP_ALL_THREAD ? fDat.CountItems():fCurPositon + DISPLAYTHREADRANGE);

	if( last > fDat.CountItems()){
		last = fDat.CountItems();
	}
	if(start == 0 && last == fDat.CountItems()){
		this->SetDispmode(DISP_ALL_THREAD);
	}


	PRINT(("pos=%ld count=%ld mode=%ld\n", start, last, Dispmode()));
	MakeSelectable(false);

	for (int32 i = start; i < last && !fCancel; i++) {
		fThread.push_back(this->TextLength());
		if (fThread_id) Window()->Lock();
		this->InsertThread(i, this);
		if (fThread_id) Window()->Unlock();
		char msg[32];
		sprintf(msg, "Insert %ld", i + 1);
		this->PostStatus(msg);
	}

	this->ScrollToRes(fCurPositon + 1);
	MakeSelectable(true);
}
////////////////////////////////////////////////////////////////////////////////
// InsertThread
//
//
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::InsertThread(int32 i, Base2chTextView *outView)
{
	int32 argc;
	const char *argv[6];
	this->fDat.OffsetAt(i, &argc, argv);
	
	//20060125:エラー対策
	if (!argv[1]) return;
	
	//スレ番
	outView->Insert(itoa(i+1), MStyle::BOLD());
	outView->Insert(" : ", MStyle::PLAIN());
	//メール欄
	if (*argv[1]) outView->SetInvoke(argv[1], strlen(argv[0]));
	//なまえ
	outView->Insert(argv[0], (*argv[1] ? MStyle::SAGE() : MStyle::AGE()));
	outView->Insert(" : ", MStyle::PLAIN());
	//日付
	outView->Insert(argv[2]);
	outView->Insert("\n");
	//本文
	outView->HtmlParser(argv[3]);
	outView->Insert("\n\n");
}

////////////////////////////////////////////////////////////////////////////////
// ThreadSelection
//
//
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::ThreadSelection(BPoint where, int32 &start, int32 &end)
{
	start = 0;
	int32 finish = 0;
	int32 offset = BTextView::OffsetAt(where);
	int32 count = fThread.size();
	for (int32 i = 1; i < count; i++) {
		finish = fThread[i];
		if (start <= offset && offset < finish) {
			end = finish;
			return;
		}
		start = finish;
	}
	end = TextLength();
	start = (offset < end) ? finish : offset;
}
////////////////////////////////////////////////////////////////////////////////
// Clone
//
// 20060320:修正
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::Clone(int32 thread_no)
{
	printf("thread no%ld\n", thread_no);

	if (fTipStiker){
		BMessenger(fTipStiker).SendMessage(B_QUIT_REQUESTED);
		fTipStiker = NULL;		
	}

	BRect frame(0, 0, 2000, 2000);
	frame.right += 6;
	frame.bottom += 6;

	K2chTipTextView *clone = new K2chTipTextView(frame, "", frame.InsetByCopy(3, 3), ResizingMode(), Flags());
	this->InsertThread(thread_no - 1, clone);

	float r = 0, b = 0;
	for (int32 i = clone->LineAt(0); i < clone->LineAt(clone->TextLength()); i++) {
		b += clone->LineHeight(i);
		if (r < clone->LineWidth(i)) r = clone->LineWidth(i) + 6;
	}

	printf("r = %f b = %f\n", r, b);
	frame = BRect(0, 0, r - 1, b - 1);
	clone->ResizeTo(frame.Width(), frame.Height());
	clone->SetViewColor(255, 253, 251);

	fTipStiker = new MSticker(frame, clone);
	fTipStiker->ShowSticker(ConvertToScreen(fStatic_where));

}
////////////////////////////////////////////////////////////////////////////////
// UpdateThreadNo
//
//
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::UpdateThreadNo(BFile *file)
{
int32 attrThreadCount = 0;
int32 count = fDat.CountItems();

	PostStatus("update Attribute");
	file->ReadAttr(DAT_ATTR_THREADCOUNT, B_INT32_TYPE, 0, &attrThreadCount, sizeof(int32));
//	ScrollToRes(attrThreadCount);

	MakeFocus(true); /* 必然性なし。利便性がいいので*/
		//スレの読込みが完了したときのみレス数を更新する。
		//TODO:読込みが完了しない場合はレス数の所得はどうするか？
		//一度に全部読込むので問題なし(20060701)
		char buf[16];
		ssize_t size = file->ReadAttr(DAT_ATTR_DATNAME, B_STRING_TYPE, 0, &buf, sizeof(buf));
		buf[size] = '\0';
		BMessage m(kMSG_UPDATE_THREADCOUNT);
		m.AddString("dat", buf);
		m.AddInt32("thread_no", count);
		BMessenger(this->Window()).SendMessage(&m);
		file->WriteAttr(DAT_ATTR_THREADCOUNT, B_INT32_TYPE, 0, &count, sizeof(int32));
		
	this->PostStatus("");
}
////////////////////////////////////////////////////////////////////////////////
// AddMenuItems
//
//
////////////////////////////////////////////////////////////////////////////////
void
K2chTextView::AddMenuItems(BPopUpMenu *menu, BPoint where)
{
	int32 start, end;
	BMessage *msg;
	
	this->GetSelection(&start, &end);
	if (start < end) {
		msg = new BMessage(kMSG_GOOGLE);
		BString data;
		GetText(start, end - start, data.LockBuffer(end - start + 1));// const
		data.UnlockBuffer();
		msg->AddString("selected", data/*.String()*/); //20060407
		menu->AddItem(new BMenuItem("Googleで検索。", msg));
		menu->AddItem(MakeItem(this, "Copy", B_COPY)); 
		menu->AddSeparatorItem();
	}
	
	menu->AddItem(MakeItem(this, "最新100スレ表示。", kMENU_THREAD_GO_LAST100)); 
	menu->AddItem(MakeItem(this, "最初から100スレ表示。", kMENU_THREAD_GO_TOP100)); 

	if(Dispmode() == DISP_PARTIAL_THREAD){
		menu->AddItem(MakeItem(this, "前100スレ表示。", kMENU_THREAD_GO_PREV100)); 
		menu->AddItem(MakeItem(this, "次100スレ表示。", kMENU_THREAD_GO_NEXT100)); 
	}

	menu->AddItem(MakeItem(this, "全スレ表示。", kMENU_THREAD_SHOW_All)); 

	this->ThreadSelection(where, start, end);
	
	if (start < end) {
		msg = new BMessage(kMSG_COPY_THREAD);
		BString data;
		GetText(start, end - start, data.LockBuffer(end - start + 1));// const
		data.UnlockBuffer();
		//BString data(Text() + start, end - start);
		msg->AddString("text", data/*.String()*/);
		menu->AddItem(new BMenuItem("Copy Thread", msg));
	}

	action_info* action = fAction.ItemTo(OffsetAt(where));
	
	if (action) {
		msg = new BMessage(kMSG_COPY_LOCATION);
		msg->AddString("text", action->name);
		menu->AddItem(new BMenuItem("Copy Link Location", msg));
	}
}
