//-------------------------------------------------------------------------------------------------------
//      Copyright (c) 2005, Be Nanashi-San 
// 
//      Permission is hereby granted, free of charge, to any person obtaining a 
//      copy of this software and associated documentation files (the "Software"), 
//      to deal in the Software without restriction, including without limitation 
//      the rights to use, copy, modify, merge, publish, distribute, sublicense, 
//      and/or sell copies of the Software, and to permit persons to whom the 
//      Software is furnished to do so, subject to the following conditions: 
// 
//      The above copyright notice and this permission notice shall be included in 
//      all copies or substantial portions of the Software. 
// 
//      THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
//      IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
//      FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
//      AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//      LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//      FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
//      DEALINGS IN THE SOFTWARE. 
// 
//      File Name:   	MUtils.cpp
//      Author:      	Be Nanashi-San@598 in "BeOS - Zeta R4" Thread 
//						(http://pc8.2ch.net/test/read.cgi/os/1120676818/598)
//		Description:	MStyle, MActionList 小物Class.
//-------------------------------------------------------------------------------------------------------
#include <Debug.h>
#include <stdio.h>

#include "MUtils.h"

// ==M=S=t=y=l=e======================================
//20051103 インライン関数としてヘッダに移動。

	BFont MStyle::thread_font;

// ==M=A=c=t=i=o=n=l=i=s=t============================
//2006.3.25 list > vector
//2006.3 clone() sakujyo
/////////////////////////////////////////////////////////////////////////////////
// MakeEmpty
//
//
////////////////////////////////////////////////////////////////////////////////
MActionList::MActionList(BTextView *target)
:fTarget(target)
, fAction(NULL), fStart(0), fFinish(0)
{
	fArray.reserve(2048);
}

MActionList::~MActionList()
{
	for(vector<action_info*>::iterator i = fArray.begin(); i != fArray.end(); i++){
		delete *i;
	}
}
/////////////////////////////////////////////////////////////////////////////////
// Clear
//
//
////////////////////////////////////////////////////////////////////////////////
void
MActionList::Clear()
{
	for(vector<action_info*>::iterator i = fArray.begin(); i != fArray.end(); i++){
		delete *i;
	}
	
	fArray.clear();
	fArray.reserve(2048);
	fStart = 0;
	fFinish = 0;
	fAction = NULL;
}
////////////////////////////////////////////////////////////////////////////////
// AddItem
//
//
////////////////////////////////////////////////////////////////////////////////
bool
MActionList::AddAction(action_info *action)
{
	fArray.push_back(action);
	fFinish = fArray.size();	/* 保険 */
	return true;
}
////////////////////////////////////////////////////////////////////////////////
// ItemTo
// ItemTo:offsetの範囲に含まれるactionを返す
//	MouseMovedで頻繁に呼び出されるのでfStart、fFinishで限定されるのは効果が期待される
//	ただ適切に更新されないと『あるのに見つからない』ということになりかねない 
////////////////////////////////////////////////////////////////////////////////

action_info*
MActionList::ItemTo(int32 offset)
{
	if (fArray.empty()) return NULL;

	if (fAction && fAction->offset <= offset && offset <= fAction->offset + fAction->length)
		return fAction;

	if (fFinish <= 0) fFinish = fArray.size();
	
	for (int32 i= fStart; i < fFinish; i++) {
		fAction = fArray[i];
		if (fAction->offset <= offset && offset <= fAction->offset + fAction->length)
			return fAction;
	}
	return NULL;
}

////////////////////////////////////////////////////////////////////////////////
// SetBounds
//
//
////////////////////////////////////////////////////////////////////////////////
void
MActionList::SetBounds(BRect frame)
{
	int32 count = fArray.size();
	fStart = 0; fFinish = count;
	for (int32 i = 0; i < count; i++) {
		action_info* action = fArray[i];
		if (frame.top > fTarget->PointAt(action->offset).y) fStart = i + 1;
		/* topより小さいということは含まれないということなので次(+1)が対象になるから */

		float height;
		BPoint point = fTarget->PointAt(action->offset + action->length, &height);
		if (frame.bottom > point.y-height) fFinish = i + 1;
	}
}
////////////////////////////////////////////////////////////////////////////////
// PrintToStream
//
//
////////////////////////////////////////////////////////////////////////////////
void
MActionList::PrintToStream()
{
	printf("<MActionList(%ld)\n", fArray.size());
	int32 count = fArray.size();
	for (int32 i = 0; i < count; i++) {
		action_info* action = fArray[i];
		printf(" %5ld  %ld %ld:%s\n", action->offset, action->length, action->size, action->name);
	}
	printf("MActionList>\n");
}



// ==G=o=o=g=l=e=U=R=L=====================================
/////////////////////////////////////////////////////////////////////////////////
// GoogleURL
// 引数serchで表される文字列をGoogole検索するURLを生成。
// 2005.10.25:BStringで書き直し。
// 2005.11.15:ヘッダーに移動。
//
// Assign("&as_sitesearchg", "hoge.com"); この場合btnG=検索となるのか (サイト内検索)
// Assign("&hq", "inurl:hoge.com/pub"); この場合btnG=検索となるのか (サイト内ディレクトリー検索)
////////////////////////////////////////////////////////////////////////////////
