#ifndef		_MUtils_
#define		_MUtils_
//-------------------------------------------------------------------------------------------------------
//      Copyright (c) 2005, Be Nanashi-San 
// 
//      Permission is hereby granted, free of charge, to any person obtaining a 
//      copy of this software and associated documentation files (the "Software"), 
//      to deal in the Software without restriction, including without limitation 
//      the rights to use, copy, modify, merge, publish, distribute, sublicense, 
//      and/or sell copies of the Software, and to permit persons to whom the 
//      Software is furnished to do so, subject to the following conditions: 
// 
//      The above copyright notice and this permission notice shall be included in 
//      all copies or substantial portions of the Software. 
// 
//      THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
//      IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
//      FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
//      AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//      LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//      FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
//      DEALINGS IN THE SOFTWARE. 
// 
//      File Name:   	MUtils.h
//      Author:      	Be Nanashi-San@598 in "BeOS - Zeta R4" Thread 
//						(http://pc8.2ch.net/test/read.cgi/os/1120676818/598)
//		Description:	MStyle, MActionList 小物Class.
//-------------------------------------------------------------------------------------------------------

#include <Font.h>
#include <String.h>
#include <TextView.h>		// for text_run_array
#include <vector>		// for text_run_array
#include "Util.h"

struct style_info {
	int32 offset;
	int32 id;
};


struct layout_info {
	int32 offset;
	BRect frame;
};

struct action_info {
	action_info():name(NULL){};
	~action_info(){if (name) free(const_cast<char *>(name));};
	
	int32 offset;			// actionのoffset位置
	int32 length;			// offset位置からのlength(actionの有効範囲)
	const char *name ;		// action名
	int32 size;				// nameの長さ
};

//20051103
// runが１個だけのtext_run_array
struct text_run_one{
	public:
	text_run_one(const BFont *font, const uint8 r, const uint8 g, const uint8 b)
	{
		count = 1;
		run.offset = 0;
		run.font = BFont(font);
		run.color.red = r; run.color.green = g; run.color.blue = b; run.color.alpha = 0;
	};

	int32 count;
	text_run run;
};

// ==M=S=t=y=l=e======================================
//20051103
//もっとすっきりできないか？
class MStyle
{
	public:
	
	static const text_run_array* PLAIN(){
		static text_run_one plain(be_plain_font, 0, 0, 0);
		return reinterpret_cast<const text_run_array*>(&plain);
	}

	static const text_run_array* BOLD(){
		static text_run_one bold(be_bold_font, 0, 0, 0);
		return reinterpret_cast<const text_run_array*>(&bold);
	}

	static const text_run_array* LINK(){
		static text_run_one line(be_plain_font, 0, 0, 255);
		return reinterpret_cast<const text_run_array*>(&line);
	}

	static const text_run_array* SAGE(){
		static text_run_one sage(be_plain_font, 0, 0, 255);
		return reinterpret_cast<const text_run_array*>(&sage);
	}

	static const text_run_array* AGE(){
		static text_run_one age(be_plain_font, 0, 128, 0);
		return reinterpret_cast<const text_run_array*>(&age);
	}

	void SetFont( float size){
		
	}
	private:
	MStyle(){}
	
	static BFont thread_font;
	
};


// ==M=A=c=t=i=o=n=l=i=s=t============================
class BTextView;
class MActionList{
	public:
	MActionList(BTextView *target);
	~MActionList();
	void Clear();
	bool AddAction(action_info*);
	action_info* ItemTo(int32 offset);
	void SetBounds(BRect frame);
	const action_info* Current() { return fAction;};
	void PrintToStream();

	private:
	BTextView*	fTarget;
	action_info *fAction;
	int32 fStart, fFinish;
	vector<action_info *> fArray; 
};


// ==G=o=o=g=l=e=U=R=L=====================================
class GoogleURL
{
	public:
	GoogleURL(const char *search);
	const char*	String();

	private:
	BString fUrl;
	GoogleURL();
};

inline 
GoogleURL::GoogleURL(const char *search)
:fUrl("http://www.google.co.jp/search?hl=ja&ie=Shift_JIS&q=")
{
	Urlescape(search, fUrl);
	fUrl += "&lr=lang_ja";
}

inline const char*
GoogleURL::String()
{
	return fUrl.String();
}

#endif
