//------------------------------------------------------------------------------ 
//      Copyright (c) 2004, Be Nanashi-San 
// 
//      Permission is hereby granted, free of charge, to any person obtaining a 
//      copy of this software and associated documentation files (the "Software"), 
//      to deal in the Software without restriction, including without limitation 
//      the rights to use, copy, modify, merge, publish, distribute, sublicense, 
//      and/or sell copies of the Software, and to permit persons to whom the 
//      Software is furnished to do so, subject to the following conditions: 
// 
//      The above copyright notice and this permission notice shall be included in 
//      all copies or substantial portions of the Software. 
// 
//      THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
//      IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
//      FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
//      AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//      LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//      FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
//      DEALINGS IN THE SOFTWARE. 
// 
//      File Name:	dat2html.cpp
//      Author: 	Be Nanashi-San 
//      Description:	 
//------------------------------------------------------------------------------
#include <stdio.h>
#include <string.h>
#include <DataIO.h>
#include <File.h>
#include <String.h>
#include <TypeConstants.h>
#include "dat2html.h"
#include "datattr.h"

//リンクの変換。
static int32 ttp2href( BString& rsSrc, BString& rsDest );

/////////////////////////////////////////////////////////////////////////////////
// int32 dat2html(BFile &in, BFile &out)
// DATファイルをhtmlに変換。スレ数を返す。
/////////////////////////////////////////////////////////////////////////////////
int32
dat2html(BFile &in, BFile &out)
{ 
	char buf[256];
	int32 tn = 0; //sureban
	char name[64];//投稿者名（含むトリップ）
	char mail[64];//メールアドレス
	char date[16];//日付
	char id[32];//ID
	char *p;
	int n;

	off_t size;
	in.GetSize(&size);
	char *buf1 = new char[size];
	const char *s = buf1;
	const char *se = s + size;   
	in.Read(buf1, size);
	BMallocIO tmp;
	
	
	//print html header
	tmp.Write("<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=Shift_JIS\"></head><body bgcolor=\"white\">", 112);
	
	while(s < se){
		//read name
		p = name;
		for(;;){
			if(*s == '<' && *(s+1) == '>'){
				s += 2;
				break;
			}
				*p++ = *s++;
		}
		*p = 0;
		
		//mail addr
		p = mail;
		if(*s=='<' && *(s+1)=='>'){
			*p = 0;
		}else{
			while( *s != '<' && *(s+1) != '>') *p++ = *s++;
			*p = 0;
		}
		s+=2; //skip <>
		
		//date
		p = date;
		//Zeta 2スレの546：Be名無しさんより日付フォーマットチェック
		if (s[2] == '/' && s[5] == '/' && s[8] == ' ' && s[11] == ':'){
			for(int i = 0; i < 14; i++) *p++ = *s++; //YY/MM/DD HH:MM
		}
		*p = 0;
		
		//ID
		p = id;
		if(*s=='<' && *(s+1)=='>'){
			*p = 0;
		}else{
			s++; //skip "space"
			while( *s != '<' && *(s+1) != '>') *p++ = *s++;
			*p = 0;
		}
		s+=2; //skip <>
		
		//ヘッダー出力
		tn++;
		if (mail[0]){
			n = sprintf(buf, "<p>%ld : <a name=\"%ld\"></a><a href=\"mailto:%s\"><b>%s</b></a> : %s %s<br>", tn, tn, mail, name, date, id);
		}else{
			n = sprintf(buf, "<p>%ld : <a name=\"%ld\"></a><font color=green><b>%s</b></font> : %s %s<br>", tn, tn, name, date, id);
		}
		tmp.Write(buf, n);
	
		
		//本文
		for(;;){
			const char *ss = s;
			int i = 0;
			if(*s =='<'){
				if(*(s + 1)=='>'){
					s += 2;
					break;
				}else{
					s += 2;
					i = 2;
				}
			}
			while(*s != '<'){s++; i++;}
		BString bsFrom;
		BString bsTo;
		bsFrom.Append( ss, i+5 );
		int32 n = ttp2href( bsFrom, bsTo );
			if ( n == 0 )
				tmp.Write(ss, i);
			else
			{
				char	*pc = bsTo.LockBuffer( bsTo.Length() );
				tmp.Write(pc, bsTo.Length());
				bsTo.UnlockBuffer();
			}
		}
		//honbun end
		while(*s++ != 10){;}
		
		tmp.Write("</p>", 4);
		
	} // end of while

	//print fooder
	tmp.Write("</body></html>", 14);
	
	delete[] buf1;
	out.Write(tmp.Buffer(), tmp.Position());
	in.WriteAttr(DAT_ATTR_THREADNUM, B_INT32_TYPE, 0, &tn, sizeof(int32));

	return tn;
}

/////////////////////////////////////////////////////////////////////////////////
// ttp2href( BString& rsSrc, BString& rsDest )
// リンクを変換する。
// Zeta 2スレ571 ：Be名無しさんより
/////////////////////////////////////////////////////////////////////////////////
// TODO: プロキシ？(ime.nuなど)を挟む？（設定で対応？）
// 開いた後のNet+の文字コードをどうするか？（別ウインドウで開く？）
int32
ttp2href( BString& rsSrc, BString& rsDest )
{
	bool	bTtp = false;
	int	nResult = 0;
	BString	sTmp;
	int		nStart, nEnd;
	
	sTmp = rsSrc;
	while ( true )
	{
		bTtp = false;
		nStart = sTmp.IFindFirst( "http://" );
		if ( nStart == B_ERROR )
		{
			nStart = sTmp.IFindFirst( "ttp://" );
			if ( nStart == B_ERROR )
				break;
			bTtp = true;
		}
		nResult++;
		
		rsDest.Append( sTmp, nStart );
		sTmp.Remove( 0, nStart );
		
		nEnd = sTmp.IFindFirst( "<br>" );
		bool bBR = true;
		if ( nEnd == B_ERROR )
		{
			nEnd = sTmp.IFindFirst( "<>" );
			if ( nEnd == B_ERROR )
				break;
			bBR = false;
		}
		rsDest.Append( "<a href = " );
		if ( bTtp )
			rsDest.Append( "h" );
		rsDest.Append( sTmp, nEnd );
		rsDest.Append( ">" );
		rsDest.Append( sTmp, nEnd );
		rsDest.Append( "</a>" );
		if ( bBR )
			sTmp.Remove( 0, nEnd+4 );
		else
			sTmp.Remove( 0, nEnd+2 );
	}
	return nResult;	
}


