//------------------------------------------------------------------------------ 
//      Copyright (c) 2005, Be Nanashi-San 
// 
//      Permission is hereby granted, free of charge, to any person obtaining a 
//      copy of this software and associated documentation files (the "Software"), 
//      to deal in the Software without restriction, including without limitation 
//      the rights to use, copy, modify, merge, publish, distribute, sublicense, 
//      and/or sell copies of the Software, and to permit persons to whom the 
//      Software is furnished to do so, subject to the following conditions: 
// 
//      The above copyright notice and this permission notice shall be included in 
//      all copies or substantial portions of the Software. 
// 
//      THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
//      IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
//      FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
//      AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//      LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//      FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
//      DEALINGS IN THE SOFTWARE. 
// 
//      File Name:	PrefWindow.cpp
//      Author: 	Be Nanashi-San 
//      Description:	 
//------------------------------------------------------------------------------
/*
	非互換PrefWindow.cpp
	NetPositiveライク設定Window
	生成で2chPositive/settingsからフラットなBMessageを読み込み、廃棄で保存する
	PrefWindowを利用する側は名前とタイプを知っていると仮定する。
	設定値はMessage()でBMessage*を受け取り、BMessageのFindxxx()関数を使って値を受け取る
	>> fPrefWindow->Message()->FindRect(kSETTING_WINDOW_RECT);
	PrefWindowで値が変更されて適用、またはOKでkUPDATEPREFSのBMessageをbe_appが受け取る。
	アプリケーションから値を設定するにはkUPDATEPREFSのBMessageにして送る。
		が面倒くさいかも。PrefWindowにReplacexxx()のような関数を作って直接更新する手もあるが
		Lock()、Unlock()が必要になる
*/

#include <interface/TabView.h>
#include <Message.h>
#include <Button.h>
#include <RadioButton.h>
#include <TextControl.h>
#include <MenuField.h>
#include <MenuItem.h>
#include <Box.h>
#include <PopUpMenu.h>
#include <RadioButton.h>
#include <ScrollBar.h>
#include <interface/StringView.h>
#include <Beep.h>
#include <stdio.h>
#include <stdlib.h>
#include <ClassInfo.h>

#include "2chPositive.h"
#include "PrefWindow.h"
#include "messages.h"
#include "TSettings.h"
#include "setting.h"
#include "Util.h"

static const uint32 kUPDATEPREFS = 'kPRF';
static const uint32 kOKBUTTON = 'kOKB';
static const uint32 kCONTROL_CHANGED  = 'kDPY';
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
inline BRect
IEtoBRect(float x, float y, float w, float h)
{
	return  BRect(x, y, x + w - 1 , y + h - 1 );
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
static BPopUpMenu*
CreateMenuFontsList(const char *name)
{
	BPopUpMenu *menu = new BPopUpMenu("");
	int32 numFamilies = count_font_families();
	for (int32 i = 0; i < numFamilies; i++) {
		font_family family;
		uint32 flags;
		if (get_font_family(i, &family, &flags) == B_OK) {
			BMessage *m = new BMessage(kCONTROL_CHANGED);
			m->AddString("name", name);
			menu->AddItem(new BMenuItem(family, m));
		}
	}

	return menu;
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
static BPopUpMenu*
CreateMenuStringList(const char *name)
{
	const char *fontsize[] = { "9", "10", "11", "12", "13", "14", "16", "18", "20", "24", NULL };
	const char **argv = fontsize;
	BPopUpMenu *menu = new BPopUpMenu("");
	while (*argv) {
		BMessage *m = new BMessage(kCONTROL_CHANGED);
		m->AddString("name", name);
		menu->AddItem(new BMenuItem(*argv, m));
		argv++;
	}
	return menu;
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
/* BMenuFieldはそのViewとmenuを表示するための_mc_mb_Viewから構成される。
	Viewの大きさはframeで固定ではなくmenu表示に必要な大きさで増減する。
	最大値としては意味がある */
static void
CreateMenuField(BRect frame, const char *label, BMenu *menu, BView *v)
{
	BMenuField* aMenuField = new BMenuField(frame, "mField", label, menu);
	aMenuField->SetDivider(110.0);
	aMenuField->SetAlignment(B_ALIGN_RIGHT);
	v->AddChild(aMenuField);
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
void
PrefWindow::SetMarkeMenuField(BMenuField *target, const char *value)
{

	const char *s = fClone->FindString(value);
	if (s && *s) { /* NULLだとFindItem()で落ちるので */
		BMenuItem* menuItem = target->Menu()->FindItem(s);
		if (menuItem) menuItem->SetMarked(true);
	}
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
static void
MakeRadioButton(BRect r,  const char *name, const char *label, uint32 command, BView *view)
{
	BRadioButton* r1 = new BRadioButton(r, name, label, new BMessage(command));
	view->AddChild(r1);
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
void
PrefWindow::SetRadioButton(BRadioButton *target, const char *value)
{
	const char *s = fClone->FindString(value);
	if (s && *s) { /* NULLだとFindItem()で落ちるので */
		int32 i = atoi(s);
		target->SetValue(i);
	}
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
static void
MakeTextField(BRect r,  const char *name, const char *label, uint32 command, float divide, BView *view)
{
	BTextControl* aTextControl1 = new BTextControl(r, name, label, "", new BMessage(command));
	aTextControl1->SetAlignment(B_ALIGN_RIGHT, B_ALIGN_LEFT);
	aTextControl1->SetDivider(divide);
	view->AddChild(aTextControl1);
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
static BButton*
MakeButton(BRect r, const char *label, uint32 command, BView *view, bool b = true)
{
	BButton* aButton = new BButton(r, "", label, new BMessage(command), B_FOLLOW_RIGHT|B_FOLLOW_BOTTOM);
	aButton->SetEnabled(b);
	view->AddChild(aButton);
	return aButton;
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
void
PrefWindow::SetTextField(BTextControl *target, const char *value)
{
	const char *s = fClone->FindString(value);
	if (s && *s) { 
		target->SetText(s);
	}
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
DisplayPrefs::DisplayPrefs(BRect frame)
	: BView(frame, "表示", B_FOLLOW_ALL, B_WILL_DRAW)
{
	SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	
	CreateMenuField(IEtoBRect(10, 10, 700, 25), "板一覧フォント:", CreateMenuFontsList(DISPLAY_BOARD_FONT_NAME), this);
	CreateMenuField(IEtoBRect(220, 10, 700, 25), "サイズ:", CreateMenuStringList(DISPLAY_BOARD_FONT_SIZE), this);
	CreateMenuField(IEtoBRect(10, 35, 700, 25), "スレ一覧フォント:", CreateMenuFontsList( DISPLAY_RESLIST_FONT_NAME), this);
	CreateMenuField(IEtoBRect(220, 35, 700, 25),"サイズ:", CreateMenuStringList(DISPLAY_RESLIST_FONT_SIZE), this);
	CreateMenuField(IEtoBRect(10, 60, 700, 25), "スレフォント:", CreateMenuFontsList(DISPLAY_RES_FONT_NAME), this);
	CreateMenuField(IEtoBRect(220, 60, 700, 25), "サイズ:", CreateMenuStringList(DISPLAY_RES_FONT_SIZE), this);

	AddChild(new BBox(IEtoBRect(10, 90, 380, 2))); 

	AddChild(new BStringView(IEtoBRect(10, 100, 300, 20),"","スレを開いたときの表示方法"));
	
	MakeRadioButton(IEtoBRect(20, 120, 300, 20), "r", "1から表示量分のレスを表示", kCONTROL_CHANGED, this);
	MakeRadioButton(IEtoBRect(20, 140, 300, 20), "r", "最後から表示量分のレスを表示", kCONTROL_CHANGED, this);
	MakeRadioButton(IEtoBRect(20, 160, 300, 20), "r", "既読から表示量分のレスを表示", kCONTROL_CHANGED, this);

	MakeTextField(IEtoBRect(10, 190, 130, 20), GENERAL_PORT, "レス表示量：", kCONTROL_CHANGED, 90.0, this);
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
void
DisplayPrefs::AttachedToWindow()
{
	const char *s[]={DISPLAY_BOARD_FONT_NAME,DISPLAY_BOARD_FONT_SIZE,
				DISPLAY_RESLIST_FONT_NAME,DISPLAY_RESLIST_FONT_SIZE,
				DISPLAY_RES_FONT_NAME,DISPLAY_RES_FONT_SIZE};
	
	PrefWindow *w = ((PrefWindow*)Window());

	for(int32 i = 0; i < 6; i++){			
		w->SetMarkeMenuField((BMenuField*)ChildAt(i), (s[i]));
	}

	w->SetRadioButton((BRadioButton*)ChildAt(8), DISPLAY_ALL_THREAD);
	w->SetRadioButton((BRadioButton*)ChildAt(9), DISPLAY_FIRST_THREAD);
	w->SetRadioButton((BRadioButton*)ChildAt(10), DISPLAY_LAST_THREAD);

	w->SetTextField((BTextControl*)ChildAt(11), DISPLAY_THREAD_RANGE);
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
ConnectionPrefs::ConnectionPrefs(BRect frame)
	: BView(frame, "接続", B_FOLLOW_ALL, B_WILL_DRAW)
{
	SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	BMenu *aMenu = new BPopUpMenu(GENERAL_CHANNEL);
	aMenu->AddItem(new BMenuItem("2ch", new BMessage(kCONTROL_CHANGED)));
	aMenu->AddSeparatorItem();
	aMenu->AddItem(new BMenuItem("Add BBS"B_UTF8_ELLIPSIS, new BMessage(kCONTROL_CHANGED)));

	BMenuField* aMenuField = new BMenuField(IEtoBRect(30,10,250,25), "BBS", "BBS:", aMenu);
	aMenuField->SetAlignment(B_ALIGN_RIGHT);
	aMenuField->SetDivider(90.0);
	AddChild(aMenuField);

	MakeTextField(IEtoBRect(30,50,260,20), GENERAL_LINKPROXY, "外部リンク用：", kCONTROL_CHANGED, 90.0, this);
	MakeTextField(IEtoBRect(30,80,260,20), GENERAL_BBSURL, "板一覧のURL：", kCONTROL_CHANGED, 90.0, this);
	MakeTextField(IEtoBRect(30,110,260,20), GENERAL_PROXY, "プロキシ：", kCONTROL_CHANGED, 90.0, this);
	MakeTextField(IEtoBRect(30,140,130,20), GENERAL_PORT, "ポート：", kCONTROL_CHANGED, 90.0, this);
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
void
ConnectionPrefs::AttachedToWindow()
{
	PrefWindow *w = ((PrefWindow*)Window());
	w->SetMarkeMenuField((BMenuField*)ChildAt(0), GENERAL_CHANNEL);

	const char *name[] = { GENERAL_LINKPROXY, GENERAL_BBSURL, GENERAL_PROXY, GENERAL_PORT };
	for (int32 i= 1; i < 5; i++)
		w->SetTextField((BTextControl *)this->ChildAt(i), name[i-1]);
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
CachePrefs::CachePrefs(BRect frame)
	: BView(frame, "キャッシュ", B_FOLLOW_ALL, B_WILL_DRAW)
{
	SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	MakeTextField(IEtoBRect(30,10,360,20), GENERAL_LINKPROXY, "キャッシュのパス：", kCONTROL_CHANGED, 120.0, this);
	MakeButton(IEtoBRect(150,70,80,27), "今すぐ削除", kMSG_CLEAR_ALL_CACHE, this);
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
void
CachePrefs::AttachedToWindow()
{
}
////////////////////////////////////////////////////////////////////////////////
// コンストラクタ
//
//
////////////////////////////////////////////////////////////////////////////////
PrefWindow::PrefWindow()
	:	Dialog(IEtoBRect(300,200,400,300), "2ch+設定")
	, fChanged(false)
	, fClone(NULL)
{
	this->CloneMessage();
	BView * contentView = new BView(this->Bounds(), "ContentView", B_FOLLOW_ALL, B_WILL_DRAW);
	contentView->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	this->AddChild(contentView);
	
	BRect frame(Bounds());
	float left = frame.right - 9 - 106 - 106 - 80;
	float top = frame.bottom - 27 - 9;
	MakeButton(IEtoBRect(left,top,80,27), "キャンセル", kOKBUTTON, contentView);
	MakeButton(IEtoBRect(left+106,top,80,27), "適用", kOKBUTTON, contentView, false);
	BButton *aButton3 = MakeButton(IEtoBRect(left+106+106,top,80,27), "OK", kOKBUTTON, contentView, false);
	this->SetDefaultButton(aButton3); /* BTextControlでEnterされるとDefaultButtonが受け取ってしまいBTextControlから
																BMessageが飛んで来ない */
	frame.top = 10;
	frame.bottom -= 44;
	fTabView = new BTabView(frame, "TabContain", B_WIDTH_FROM_LABEL);
	frame.top = 0;
	frame.bottom -= 13 + fTabView->TabHeight();
	fTabView->AddTab(new ConnectionPrefs(frame));
	fTabView->AddTab(new DisplayPrefs(frame));
	fTabView->AddTab(new CachePrefs(frame));
	contentView->AddChild(fTabView);
}

PrefWindow::~PrefWindow()
{
}
////////////////////////////////////////////////////////////////////////////////
// MessageReceived
//
//
////////////////////////////////////////////////////////////////////////////////
void
PrefWindow::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
	case kCONTROL_CHANGED:
		ModifiedMessage(msg);
		break;
	case kOKBUTTON:
		DispatchButton(msg);
		break;
	case kUPDATEPREFS: 
	/* 設定のfMessageをmsgで更新する。直接更新するという手もあるが */
		UpdateMessage(msg);
		break;
	case kMSG_CLEAR_ALL_CACHE:
		ClearBBSCache();
		break;
	default:
		BWindow::MessageReceived(msg);
		break;
	}
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
bool
PrefWindow::QuitRequested()
{
	bool internal = CurrentMessage();
	if (internal)
		this->Hide();
	else
		this->DeleteCloneMessage();
	
	return ! internal;
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
/* msgからButtonを判定してSave、Quitを行う */
void
PrefWindow::DispatchButton(BMessage *msg)
{
	BView *object;
	if (msg->FindPointer("source", (void**)&object) == B_OK) {
		int32 flags = 0;
		if (ChildAt(0)->ChildAt(0) == object)
			flags = 2; 			/*更新せず終了 */
		else if (ChildAt(0)->ChildAt(1) == object)
			flags = 1;			/* 更新のみ */
		else if (ChildAt(0)->ChildAt(2) == object)
			flags = 3;			/* 更新して終了 */

		if (flags & 1) {
			BMessage *diff = DiffMessage(fClone, theSettings->Settings());
			if (! diff->IsEmpty()) be_app->PostMessage(diff);
			delete diff;

			theSettings->Clear();
		/* flags == 1ならWindowは閉じないのでfCloneを残して置かなければならないので */
			theSettings->SetTo(*fClone);
		}
		if (flags & 2) {
		/* Windowを閉じるので廃棄する。flagsが0のこともある(前提条件が違っている)ので確認する必要がある */
			DeleteCloneMessage();
			PostMessage(B_QUIT_REQUESTED);
		}
		ButtonEnable(false); /* いずれにしろ更新はクリアーされるので */
	}
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
/* msgから更新されたnameとvalueを入手してfCloneへ反映させる */
void
PrefWindow::ModifiedMessage(BMessage *msg)
{
	BArchivable *object;
	if (msg->FindPointer("source", (void**)&object) == B_OK) {
		const char *name = NULL, *value = NULL;
		if (is_instance_of(object, BMenuItem)) {
			BMenuItem* target = dynamic_cast<BMenuItem*>(object);
			name = target->Menu()->Name();
			value = target->Label();
		}
		else if (is_instance_of(object, BTextControl)) {
			BTextControl* target = dynamic_cast<BTextControl*>(object);
			name = target->Name();
			value = target->Text();
		}
		else if (is_instance_of(object, BRadioButton)) {
			BRadioButton* target = dynamic_cast<BRadioButton*>(object);
			name = target->Name();
			value = target->Value() ? "1" : "0";
			}else return;
			
		/* fCloneを更新する */
		if (fClone->HasString(name))
			fClone->ReplaceString(name, value);
		else
			fClone->AddString(name, value);

		fChanged = true;
		if (fChanged) ButtonEnable(true);
	}
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
/* fMessageのコピー(fMessageを直接変更してはならない)を作成する*/
void
PrefWindow::CloneMessage()
{
	if (fClone) delete fClone;
	fClone = theSettings->MakeCopy();
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
/* fMessageのコピー(fMessageを直接変更してはならない)を作成する*/
void
PrefWindow::DeleteCloneMessage()
{
		delete fClone;
		fClone = NULL;
		fChanged = false;
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
/* msgでfMessageを更新する。indexは考慮されない。*/
void
PrefWindow::UpdateMessage(BMessage *msg)
{
	BMessage *fMessage = theSettings->Settings();
	const char *nameFound;
	type_code typeFound;
	int32 countFound, i= 0;
	while (msg->GetInfo(B_ANY_TYPE, i++, &nameFound, &typeFound, &countFound) == B_OK) {
		const void *data = NULL;
		ssize_t numBytes = 0;
		msg->FindData(nameFound, typeFound, 0, &data, &numBytes);
		if (fMessage->HasData(nameFound, typeFound))
			fMessage->ReplaceData(nameFound, typeFound, data, numBytes);
		else {
		/* 何故かB_STRING_TYPEのものをAddData()するとPrintToStream()で変に表示される。
			FindString()では問題なく受け取れるのだが？ ReplaceDataは問題ないのか？ */
			if (typeFound == B_STRING_TYPE)
				fMessage->AddString(nameFound, (const char*)data);
			else
				fMessage->AddData(nameFound, typeFound, data, numBytes);
		}
		fChanged = true;
	}
	if (fChanged) ButtonEnable(true);
	/* 既にfCloneされていたら削除して再設定する必要がある */
	if (fClone) {
		this->CloneMessage();
		BView *active = fTabView->ViewForTab(fTabView->Selection());
		active->AttachedToWindow();	/* 呼び出してもいいのか */
	}
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
/* srcにある物でdstと異なるものを抽出する */
BMessage*
PrefWindow::DiffMessage(BMessage *src, BMessage *dst)
{
	BMessage *diff = new BMessage(kUPDATEPREFS);
	const char *nameFound;
	type_code typeFound;
	int32 countFound, t= 0;
	while (src->GetInfo(B_ANY_TYPE, t++, &nameFound, &typeFound, &countFound) == B_OK) {
		for (int32 i= 0; i < countFound; i++) {
			const void *data = NULL;
			ssize_t numBytes = 0;
			src->FindData(nameFound, typeFound, i, &data, &numBytes);
			if (dst->HasData(nameFound, typeFound, i)) {
				const void *data2 = NULL;
				ssize_t numBytes2 = 0;
				dst->FindData(nameFound, typeFound, i, &data2, &numBytes2);
				if (numBytes == numBytes2 && memcmp(data, data2, numBytes) == 0)
					continue;
			}
		/* 何故かB_STRING_TYPEのものをAddData()するとPrintToStream()で変に表示される。
			FindString()では問題なく受け取れるのだが？ */
			if (typeFound == B_STRING_TYPE)
				diff->AddString(nameFound, (const char*)data);
			else
				diff->AddData(nameFound, typeFound, data, numBytes);
	}	}
//	diff->PrintToStream();
	return diff;
}
////////////////////////////////////////////////////////////////////////////////
// 
//
//
////////////////////////////////////////////////////////////////////////////////
void
PrefWindow::ButtonEnable(bool flags)
{
	((BButton*)(ChildAt(0)->ChildAt(1)))->SetEnabled(flags);
	((BButton*)(ChildAt(0)->ChildAt(2)))->SetEnabled(flags);
}
