//------------------------------------------------------------------------------ 
//      Copyright (c) 2004, Be Nanashi-San 
// 
//      Permission is hereby granted, free of charge, to any person obtaining a 
//      copy of this software and associated documentation files (the "Software"), 
//      to deal in the Software without restriction, including without limitation 
//      the rights to use, copy, modify, merge, publish, distribute, sublicense, 
//      and/or sell copies of the Software, and to permit persons to whom the 
//      Software is furnished to do so, subject to the following conditions: 
// 
//      The above copyright notice and this permission notice shall be included in 
//      all copies or substantial portions of the Software. 
// 
//      THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
//      IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
//      FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
//      AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//      LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//      FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
//      DEALINGS IN THE SOFTWARE. 
// 
//      File Name:   StatusView.cpp
//      Author:      Be Nanashi-San 
//      Description: Status Bar	 
//------------------------------------------------------------------------------

#include <Bitmap.h>
#include <Rect.h>
#include <PictureButton.h>
#include <Messenger.h>
#include <MenuBar.h>
#include <MenuItem.h>
#include <Window.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "messages.h"
#include "StatusBarView.h"
#include "2chPositive.h"
#include "setting.h"

//Image translated by QuickRes (copy as text)
static const int32 kImageWidth = 27;
static const int32 kImageHeight = 13;

static const unsigned char kOnLineImageBits[] = {
	0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 
	0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x0e, 0x3f, 0x3f, 0x1b, 0x2a, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x2a, 0x1b, 0x1b, 0x1b, 0x1b, 0x2a, 0x1b, 0x1b, 0x1b, 0x1b, 0x2a, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x2a, 0x1b, 0x1b, 0x0f, 0x3f, 0x3f, 0x1b, 0x1b, 0x2a, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x2a, 0x1b, 0x1b, 0x1b, 0x1b, 0x2a, 0x1b, 0x1b, 0x1b, 0x1b, 0x2a, 0x1b, 0x1b, 0x1b, 0x2a, 0x1b, 
	0x1b, 0x1b, 0x0f, 0x3f, 0x3f, 0x1b, 0x1b, 0x1b, 0x2a, 0x1b, 0x04, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x04, 0x05, 0x04, 0x04, 0x04, 0x05, 0x04, 0x04, 0x1b, 0x2a, 0x1b, 0x1b, 0x1b, 0x1b, 0x10, 0x3f, 
	0x3f, 0x1b, 0x1b, 0x1b, 0x04, 0x04, 0x04, 0x1e, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 
	0x3f, 0x3f, 0x1e, 0x04, 0x04, 0x05, 0x04, 0x1b, 0x1b, 0x1b, 0x0f, 0x3f, 0x3f, 0x1b, 0x1b, 0x04, 
	0x1e, 0x1d, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x1e, 0x3f, 
	0x3f, 0x1e, 0x0e, 0x05, 0x1b, 0x1b, 0x0f, 0x3f, 0x3f, 0x1b, 0x1b, 0x04, 0x1e, 0x3f, 0x3f, 0x3f, 
	0x3f, 0x3f, 0x3f, 0x04, 0x3f, 0x3f, 0x3f, 0x04, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x04, 
	0x1b, 0x1b, 0x10, 0x3f, 0x3f, 0x1b, 0x04, 0x1e, 0x3f, 0x3f, 0x04, 0x04, 0x3f, 0x3f, 0x3f, 0x04, 
	0x05, 0x04, 0x04, 0x04, 0x3f, 0x3f, 0x3f, 0x05, 0x04, 0x3f, 0x3f, 0x1e, 0x04, 0x1b, 0x0f, 0x3f, 
	0x3f, 0x1b, 0x04, 0x1e, 0x3f, 0x3f, 0x04, 0x04, 0x3f, 0x3f, 0x3f, 0x17, 0x05, 0x7b, 0x04, 0x17, 
	0x1f, 0x1f, 0x3f, 0x06, 0x04, 0x1f, 0x3f, 0x3f, 0xd7, 0x1b, 0xa8, 0x3f, 0x3f, 0x1b, 0x04, 0x1e, 
	0x1d, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x05, 0x7b, 0xde, 0x1f, 0x1f, 0x3f, 0x3f, 0x3f, 
	0x3f, 0x3f, 0x3f, 0x3f, 0x06, 0x1b, 0x10, 0x3f, 0x3f, 0x1b, 0x04, 0x1e, 0x1d, 0x1e, 0x3f, 0x3f, 
	0x3f, 0x3f, 0x3f, 0x3f, 0x17, 0x05, 0x17, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 
	0x04, 0x1b, 0x10, 0x3f, 0x3f, 0x1b, 0x1b, 0x04, 0x1e, 0x1d, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 
	0x3f, 0x05, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x04, 0x1b, 0x1b, 0x0f, 0x3f, 
	0x3f, 0x0f, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 
	0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x3f
};

static const unsigned char kOffLineImageBits[] = {
	0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 
	0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x0e, 0x3f, 0x3f, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x0f, 0x3f, 0x3f, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x0f, 0x3f, 0x3f, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x04, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x04, 0x05, 0x04, 0x04, 0x04, 0x05, 0x04, 0x04, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x10, 0x3f, 
	0x3f, 0x1b, 0x1b, 0x1b, 0x04, 0x04, 0x04, 0x1e, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 
	0x3f, 0x3f, 0x1e, 0x04, 0x04, 0x05, 0x04, 0x1b, 0x1b, 0x1b, 0x0f, 0x3f, 0x3f, 0x1b, 0x1b, 0x04, 
	0x1e, 0x1d, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x1e, 0x3f, 
	0x3f, 0x1e, 0x0e, 0x05, 0x1b, 0x1b, 0x0f, 0x3f, 0x3f, 0x1b, 0x1b, 0x04, 0x1e, 0x3f, 0x3f, 0x3f, 
	0x3f, 0x3f, 0x3f, 0x04, 0x3f, 0x3f, 0x3f, 0x04, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x04, 
	0x1b, 0x1b, 0x10, 0x3f, 0x3f, 0x1b, 0x04, 0x1e, 0x3f, 0x3f, 0x04, 0x04, 0x3f, 0x3f, 0x3f, 0x04, 
	0x05, 0x04, 0x04, 0x04, 0x3f, 0x3f, 0x3f, 0x05, 0x04, 0x3f, 0x3f, 0x1e, 0x04, 0x1b, 0x0f, 0x3f, 
	0x3f, 0x1b, 0x04, 0x1e, 0x3f, 0x3f, 0x04, 0x04, 0x3f, 0x3f, 0x3f, 0x17, 0x05, 0x7b, 0x04, 0x17, 
	0x1f, 0x1f, 0x3f, 0x06, 0x04, 0x1f, 0x3f, 0x3f, 0xd7, 0x1b, 0xa8, 0x3f, 0x3f, 0x1b, 0x04, 0x1e, 
	0x1d, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x05, 0x7b, 0xde, 0x1f, 0x1f, 0x3f, 0x3f, 0x3f, 
	0x3f, 0x3f, 0x3f, 0x3f, 0x06, 0x1b, 0x10, 0x3f, 0x3f, 0x1b, 0x04, 0x1e, 0x1d, 0x1e, 0x3f, 0x3f, 
	0x3f, 0x3f, 0x3f, 0x3f, 0x17, 0x05, 0x17, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 
	0x04, 0x1b, 0x10, 0x3f, 0x3f, 0x1b, 0x1b, 0x04, 0x1e, 0x1d, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 
	0x3f, 0x05, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x04, 0x1b, 0x1b, 0x0f, 0x3f, 
	0x3f, 0x0f, 0x10, 0x0f, 0x10, 0x10, 0x0f, 0x10, 0x10, 0x0f, 0x10, 0x0f, 0x10, 0x10, 0x0f, 0x10, 
	0x10, 0x0f, 0x10, 0x0f, 0x10, 0x10, 0x0f, 0x10, 0x10, 0x0f, 0x10, 0x3f};

static const unsigned char kWritingImage01Bits[] = {
	0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 
	0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x0e, 0x3f, 0x3f, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x0a, 0x0b, 
	0x0a, 0x3f, 0x1b, 0x1b, 0x1b, 0x1b, 0x0f, 0x3f, 0x3f, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x0a, 0x0b, 0x3f, 0x5a, 0x00, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x0f, 0x3f, 0x3f, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x0a, 0x09, 0x5a, 0x0a, 0x5a, 0x01, 0x1b, 0x1b, 0x1b, 0x1b, 0x10, 0x3f, 
	0x3f, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x00, 0x1b, 0x1b, 0x0a, 
	0x2a, 0xd3, 0xa3, 0xa3, 0x00, 0x00, 0x1b, 0x1b, 0x1b, 0x1b, 0x0f, 0x3f, 0x3f, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x00, 0x3f, 0x00, 0x0a, 0xd3, 0xff, 0x2a, 0x08, 0xa3, 
	0x00, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x0f, 0x3f, 0x3f, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x00, 0x3f, 0x3f, 0x0a, 0x0b, 0xff, 0x2a, 0x2a, 0xd3, 0x00, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x10, 0x3f, 0x3f, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x00, 0x3f, 0x3f, 
	0x3f, 0x0a, 0x0b, 0x2a, 0x2a, 0x2a, 0x00, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x0f, 0x3f, 
	0x3f, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x00, 0x3f, 0x3f, 0x3f, 0x0a, 0x1a, 0x0b, 0x0b, 
	0x0a, 0x00, 0xfe, 0x00, 0x00, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x0f, 0x3f, 0x3f, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x00, 0x3f, 0x3f, 0x3f, 0x3f, 0x0a, 0xff, 0xd9, 0x00, 0x00, 0xfe, 0xfe, 0xfe, 
	0x00, 0x00, 0x1b, 0x1b, 0x1b, 0x1b, 0x10, 0x3f, 0x3f, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x00, 0x3f, 
	0x3f, 0x3f, 0x3f, 0x0a, 0xd9, 0xd9, 0x00, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x00, 0x1b, 
	0x1b, 0x1b, 0x10, 0x3f, 0x3f, 0x1b, 0x1b, 0x1b, 0x1b, 0x00, 0xb0, 0x3f, 0x3f, 0x3f, 0x3f, 0x00, 
	0x00, 0xb0, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x00, 0x1b, 0x1b, 0x0f, 0x3f, 
	0x3f, 0x0f, 0x10, 0x0f, 0x10, 0x10, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x10, 0x10, 0x0f, 0x10, 0x3f
};

static const unsigned char kWritingImage02Bits[] = {
	0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 
	0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x0e, 0x3f, 0x10, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x0a, 0x0b, 
	0x0a, 0x3f, 0x1b, 0x1b, 0x1b, 0x1b, 0x3f, 0x3f, 0x10, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x0a, 0x0b, 0x3f, 0x5a, 0x00, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x3f, 0x3f, 0x10, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x0a, 0x09, 0x5a, 0x0a, 0x5a, 0x01, 0x1b, 0x1b, 0x1b, 0x1b, 0x3f, 0x3f, 
	0x10, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x00, 0x1b, 0x1b, 0x0a, 
	0x2a, 0xd3, 0xa3, 0xa3, 0x00, 0x00, 0x1b, 0x1b, 0x1b, 0x1b, 0x3f, 0x3f, 0x10, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x00, 0x3f, 0x00, 0x0a, 0xd3, 0xff, 0x2a, 0x08, 0xa3, 
	0x00, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x3f, 0x3f, 0x10, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x00, 0x3f, 0x3f, 0x0a, 0x0b, 0xff, 0x2a, 0x2a, 0xd3, 0x00, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x3f, 0x3f, 0x10, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x00, 0x3f, 0x3f, 
	0x3f, 0x0a, 0x0b, 0x2a, 0x2a, 0x2a, 0x00, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x3f, 0x3f, 
	0x10, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x00, 0x3f, 0x3f, 0x3f, 0x0a, 0x1a, 0x0b, 0x0b, 
	0x0a, 0x00, 0xfe, 0x00, 0x00, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x3f, 0x3f, 0x10, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x00, 0x3f, 0x3f, 0x3f, 0x3f, 0x0a, 0xff, 0xd9, 0x00, 0x00, 0xfe, 0xfe, 0xfe, 
	0x00, 0x00, 0x1b, 0x1b, 0x1b, 0x1b, 0x3f, 0x3f, 0x10, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x00, 0x3f, 
	0x3f, 0x3f, 0x3f, 0x0a, 0xd9, 0xd9, 0x00, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x00, 0x1b, 
	0x1b, 0x1b, 0x3f, 0x3f, 0x10, 0x1b, 0x1b, 0x1b, 0x1b, 0x00, 0xb0, 0x3f, 0x3f, 0x3f, 0x3f, 0x00, 
	0x00, 0xb0, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x00, 0x1b, 0x1b, 0x3f, 0x3f, 
	0x10, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 
	0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f
};


////////////////////////////////////////////////////////////////////////////////
// StatusBarView
//
//
////////////////////////////////////////////////////////////////////////////////
StatusBarView::StatusBarView(BRect frame)
:BView(frame, "StatusBarView", B_FOLLOW_LEFT_RIGHT | B_FOLLOW_BOTTOM, B_WILL_DRAW)
,fText("2chPositiveへようこそ！")//20060414
{
}
////////////////////////////////////////////////////////////////////////////////
// ~StatusBarView
//
//
////////////////////////////////////////////////////////////////////////////////
StatusBarView::~StatusBarView()
{
}
////////////////////////////////////////////////////////////////////////////////
// MakePicture
//
//
////////////////////////////////////////////////////////////////////////////////
static BPicture *
MakePicture(BView *view, const unsigned char *data, size_t size)
{
	view->BeginPicture(new BPicture);
	BBitmap *map = new BBitmap(view->Bounds(), B_CMAP8);
	map->SetBits(data, size, 0, B_CMAP8);
	view->DrawBitmapAsync(map);
	return view->EndPicture();
}
////////////////////////////////////////////////////////////////////////////////
// AttachedToWindow
//
//
////////////////////////////////////////////////////////////////////////////////
void	
StatusBarView::AttachedToWindow()
{
	this->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	BRect r(this->Bounds());
	r.top++;
	r.bottom = r.top + kImageHeight;
	r.right = r.left + kImageWidth;

	BView *someView = new BView(BRect(0, 0, kImageWidth - 1, kImageHeight - 1), "", B_FOLLOW_NONE, B_WILL_DRAW);
	this->AddChild(someView);
	//オフライン／オンラインボタン
	BPicture *pic = MakePicture(someView, kOnLineImageBits, sizeof(kOnLineImageBits));
	BPicture *pic2 = MakePicture(someView, kOffLineImageBits, sizeof(kOffLineImageBits));
	fOnOffButton = new BPictureButton(r, "", pic, pic2, new BMessage(kMEG_ONOFF_BUTTON), B_TWO_STATE_BUTTON, B_FOLLOW_LEFT | B_FOLLOW_BOTTOM);
	fOnOffButton->SetTarget(this);

	if(theSettings->GetBool(kSETTING_OFFLINE)) //offline
		fOnOffButton->SetValue(B_CONTROL_ON);
	else //online
		fOnOffButton->SetValue(B_CONTROL_OFF);
	
	//書き込みボタン
	r.OffsetBy(kImageWidth + 1, 0.0);
	pic = MakePicture(someView, kWritingImage01Bits, sizeof(kWritingImage01Bits));
	pic2 = MakePicture(someView, kWritingImage02Bits, sizeof(kWritingImage02Bits));
	fWriteButton = new BPictureButton(r, "", pic, pic2, new BMessage(kMEG_WRITE_BUTTON), B_ONE_STATE_BUTTON, B_FOLLOW_LEFT | B_FOLLOW_BOTTOM);
	fWriteButton->SetTarget(this->Window());
	
	this->RemoveChild(someView);
	delete someView;
	this->AddChild(fOnOffButton);
	this->AddChild(fWriteButton);
}
////////////////////////////////////////////////////////////////////////////////
// MessageReceived
//
//
////////////////////////////////////////////////////////////////////////////////
void
StatusBarView::MessageReceived(BMessage *message)
{
	switch(message->what){
	case kMSG_STATUS_MESSAGE:
		message->FindString("message", &fText);
		this->Draw(this->Bounds());
		break;

	case kMEG_ONOFF_BUTTON:
		this->DoOnOff(false);
		break;

	case kMENU_FILE_OFFLINE:
		this->DoOnOff(true);
		break;
		
	default:
		BView::MessageReceived(message);
		break;
	}

}
////////////////////////////////////////////////////////////////////////////////
// DoOnOff
// オンライン／オフラインの切換。
//
////////////////////////////////////////////////////////////////////////////////
void
StatusBarView::DoOnOff(bool setButton)
{
	bool mode = theSettings->GetBool(kSETTING_OFFLINE) == kONLINE ? kOFFLINE : kONLINE;
	theSettings->SetBool(kSETTING_OFFLINE, mode);
	BMenuBar *mbar = this->Window()->KeyMenuBar();
	BMenuItem *item = mbar->SubmenuAt(0)->FindItem("オフライン");
	item->SetMarked(mode);		

	if(!setButton) return;

	if(mode) //offline
			fOnOffButton->SetValue(B_CONTROL_ON);
	else //online
			fOnOffButton->SetValue(B_CONTROL_OFF);
}

////////////////////////////////////////////////////////////////////////////////
// Draw
//
//
////////////////////////////////////////////////////////////////////////////////
void
StatusBarView::Draw(BRect updateRect)
{
	BRect rect = this->Bounds();

	this->SetHighColor(ui_color(B_MENU_BACKGROUND_COLOR));
	FillRect(updateRect);
	this->SetHighColor(125,125,125);
	this->StrokeLine(BPoint(rect.left, 0), BPoint(rect.right, 0));	
	this->SetHighColor(255, 255, 255);
	this->StrokeLine(BPoint(rect.left, 1), BPoint(rect.right, 1));	
	if(!fText.Length()) return;
	this->SetHighColor(0, 0, 0);
	// TODO:BString.trancate
	DrawString(fText.String(), BPoint(kImageWidth * 2 + 3, rect.bottom - 1));

}

