/*****************************************************************************/
// ShowImageSettings
// Written by Michael Pfeiffer
//
// ShowImageSettings.cpp
//
//
// Copyright (c) 2003 OpenBeOS Project
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
/*****************************************************************************/

#ifndef _Settings_H
#define _Settings_H

#include <Message.h>
#include <Locker.h>
#include <String.h>
#include <File.h>

class TSettings
{
public:
	TSettings(const char *path);
	~TSettings();

	void SetPath(const char *path);
	bool Lock();
	bool GetBool(const char* name);
	int32 GetInt32(const char* name, int32 defaultValue);
	float GetFloat(const char* name, float defaultValue);
	status_t GetPoint(const char* name, BPoint *point);
	BPoint GetPoint(const char* name);
	BRect GetRect(const char* name);
	const char* GetString(const char* name, const char* defaultValue);
	status_t GetMessage(const char* name, BMessage *msg);
	
	void SetBool(const char* name, bool value);
	void SetInt32(const char* name, int32 value);
	void SetFloat(const char* name, float value);
	void SetPoint(const char* name, BPoint point);
	void SetRect(const char* name, BRect value);
	void SetString(const char* name, const char* value);
	void SetMessage(const char* name, BMessage *msg);
	void Unlock();
	
	status_t Load();
	void Save();
	void Clear();
	void SetTo(BMessage &msg);
	BMessage *MakeCopy();
	BMessage *Settings();
	
private:
	status_t OpenSettingsFile(BFile* file, bool forReading);

	BLocker fLock;
	BMessage fSettings;
	BString fPath;
};

//20060312
extern TSettings* theSettings;

#endif //_Settings_H
