//------------------------------------------------------------------------------ 
//      Copyright (c) 2004, Be Nanashi-San 
// 
//      Permission is hereby granted, free of charge, to any person obtaining a 
//      copy of this software and associated documentation files (the "Software"), 
//      to deal in the Software without restriction, including without limitation 
//      the rights to use, copy, modify, merge, publish, distribute, sublicense, 
//      and/or sell copies of the Software, and to permit persons to whom the 
//      Software is furnished to do so, subject to the following conditions: 
// 
//      The above copyright notice and this permission notice shall be included in 
//      all copies or substantial portions of the Software. 
// 
//      THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
//      IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
//      FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
//      AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//      LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//      FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
//      DEALINGS IN THE SOFTWARE. 
// 
//      File Name:	ThreadListView.cpp
//      Author: 	Be Nanashi-San 
//      Description:	 
//------------------------------------------------------------------------------
//todo col_hostをメンバ変数に、thread_info, host_info(url_info)新設。
//update処理

#include <Alert.h>
#include <Beep.h>
#include <Directory.h>
#include <Entry.h>
#include <File.h>
#include <FindDirectory.h>
#include <Directory.h>
#include <InterfaceDefs.h>
#include <UTF8.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "2chPositive.h"
#include "ColumnTypes.h"
#include "DisplayThreadView.h"
#include "NetUtil.h"
#include "ThreadListView.h"
#include "Util.h"
#include "datattr.h"
#include "messages.h"
#include "setting.h"
#include "arrows.h"

enum col_type{
	COL_NO = 0,
	COL_STATUS, //↑↓→●
	COL_THREADNAME,
	COL_RES, // スレ数
	COL_READ,//キャッシュのスレ数
	COL_DATE,//最終更新日付
	COL_DATNAME
};

const unsigned int kMSG_THREADLIST_INVOCATION = 'MTHL';

////////////////////////////////////////////////////////////////////////////////
//  ThreadListView::ThreadListView
//
//
////////////////////////////////////////////////////////////////////////////////
ThreadListView::ThreadListView(BRect rect)
:BColumnListView(rect, "ThreadListView", B_FOLLOW_ALL, B_NAVIGABLE, B_FANCY_BORDER, false)
,fCacheDir(new BDirectory())
,fBoardUri(NULL)
,fBlueUpArrow(NULL)
,fYellowEqaulArrow(NULL)
,fRedDownArrow(NULL)
,fGrayBall(NULL)
,fStopLoading(false)
,fThread(0)
{
	this->SetSelectionMode(B_SINGLE_SELECTION_LIST);

	const rgb_color backcolor = {255,255,255,0};
	this->SetColor(B_COLOR_BACKGROUND, backcolor);
	const rgb_color selectioncolor = {152,152,152,0};
	this->SetColor(B_COLOR_SELECTION, selectioncolor);
	
	float colum_length;
	colum_length = be_plain_font->StringWidth ("No.");
	BIntegerColumn *icol = new BIntegerColumn ("No.", colum_length * 3, 0, colum_length * 3);
  	this->AddColumn (icol, COL_NO);

	this->SetSortColumn(icol, false, true);
	//20050709:990@みんなでBeOS！(Free) 氏により最大幅修正。
	//Ｓは2chTrackerより（Statusの意味か？）
	BBitmapColumn *bcol = new BBitmapColumn ("S", kArrowWidth, kArrowWidth, kArrowWidth * 4, B_ALIGN_CENTER);
  	this->AddColumn (bcol, COL_STATUS);

	colum_length = be_plain_font->StringWidth ("あ");
	BStringColumn *scol = new BStringColumn ("タイトル", colum_length * 20, 0, colum_length * 40, 0);
  	this->AddColumn (scol, COL_THREADNAME);

	colum_length = be_plain_font->StringWidth ("新着");
	icol = new BIntegerColumn ("新着", colum_length * 2, 0, colum_length * 2);
  	this->AddColumn (icol, COL_RES);

	colum_length = be_plain_font->StringWidth ("既得");
	icol = new BIntegerColumn ("既得", colum_length * 2, 0, colum_length * 2);
  	this->AddColumn (icol, COL_READ);

	colum_length = be_plain_font->StringWidth ("9999/99/99");
	scol = new BStringColumn ("最終取得", colum_length * 2, 0, colum_length * 2, 0);
  	this->AddColumn (scol, COL_DATE);

	scol = new BStringColumn (NULL, 0, 0, 0, 0);
  	this->AddColumn (scol, COL_DATNAME);

  	
	fBlueUpArrow = new BBitmap(BRect(0, 0, kArrowWidth - 1, kArrowHeight - 1),  B_CMAP8);
	fBlueUpArrow->SetBits(kBlueUpArrow, sizeof(kBlueUpArrow), 0, B_CMAP8);
	fYellowEqaulArrow = new BBitmap(BRect(0, 0, kArrowWidth - 1, kArrowHeight - 1),  B_CMAP8);
	fYellowEqaulArrow->SetBits(kYellowEqaulArrow, sizeof(kYellowEqaulArrow), 0, B_CMAP8);
	fRedDownArrow = new BBitmap(BRect(0, 0, kArrowWidth - 1, kArrowHeight - 1),  B_CMAP8);
	fRedDownArrow->SetBits(kRedDownArrow, sizeof(kRedDownArrow), 0, B_CMAP8);
	fGrayBall = new BBitmap(BRect(0, 0, kArrowWidth - 1, kArrowHeight - 1),  B_CMAP8);
	fGrayBall->SetBits(kGrayBall, sizeof(kGrayBall), 0, B_CMAP8);



  	BMessage msg;
  	if(theSettings->GetMessage(kSETTING_THREADLIST, &msg) == B_OK){
		this->LoadState(&msg);
	}

	fStopSem = create_sem(1, "ListReader_sem");
}
////////////////////////////////////////////////////////////////////////////////
// ~ThreadListView
//
//
////////////////////////////////////////////////////////////////////////////////
ThreadListView::~ThreadListView()
{
	BMessage msg;
  	
  	this->SaveState(&msg);
	theSettings->SetMessage(kSETTING_THREADLIST, &msg);
	
	delete fCacheDir;
	delete fBlueUpArrow;
	delete fYellowEqaulArrow;
	delete fRedDownArrow;
	delete fGrayBall;
	delete_sem(fStopSem);

}
////////////////////////////////////////////////////////////////////////////////
// AttachedToWindow
//
//
////////////////////////////////////////////////////////////////////////////////
void
ThreadListView::AttachedToWindow() 
{ 
	this->SetInvocationMessage(new BMessage(kMSG_THREADLIST_INVOCATION));
	this->SetTarget(this);
	BColumnListView::AttachedToWindow(); 
} 
////////////////////////////////////////////////////////////////////////////////
// MessageReceived
//
//
////////////////////////////////////////////////////////////////////////////////
void
ThreadListView::MessageReceived (BMessage * msg)
{
	switch (msg->what){
		case kMSG_OPEN_BOARD:
			this->OpenBoard(msg);
			break;

		case kMSG_THREADLIST_INVOCATION:
			this->ListInvoked();
			break;

		case kMENU_THREADLIST_REFREASH:
			if(fBoardUri.Length()){
				findcachedir(fCacheDir, kCACHE_DIR_NAME, fBoardUri.String());
				this->LoadList(false);
			}
			break;

		//検索：zeta 2 スレ644のBe名無しさんより。
		case kMSG_FIND:
			this->DoFind(msg);
			break;		

		//既読の更新：zeta 2 スレ643のBe名無しさんより。
		case kMSG_UPDATE_THREADCOUNT:
			this->DoUpdate(msg);
			break;			

		default:
			BColumnListView::MessageReceived (msg);
			break;
	}
}
////////////////////////////////////////////////////////////////////////////////
// OpenBoard
// スレ一覧の表示を行う。
// 同時に指定スレを開くことも可能。
////////////////////////////////////////////////////////////////////////////////
void
ThreadListView::OpenBoard (BMessage * msg)
{
	const char *uri;
	status_t sts = msg->FindString("uri", &uri);
	if(sts != B_OK) return;
	
	fBoardUri.SetTo(uri);
	findcachedir(fCacheDir, kCACHE_DIR_NAME, fBoardUri.String());


//	//datが指定されていれば、スレも開く。
//	const char *dat = NULL;
//	if (msg->FindString("dat")) {
//		dat = msg->FindString("dat");
//	}

	this->LoadList(theSettings->GetBool(kSETTING_OFFLINE), msg->FindString("dat"));
}
////////////////////////////////////////////////////////////////////////////////
// CurrentQuery
// zeta 2 スレ@713のBe名無しさんより。
//
////////////////////////////////////////////////////////////////////////////////
bool
ThreadListView::CurrentQuery (BMessage *msg)
{
	BRow* row = this->CurrentSelection();
	if (!row || !fBoardUri.Length()) return false;
	if(!msg) return true;
	
	msg->AddInt32("no", static_cast<BIntegerField *>(row->GetField(COL_NO))->Value());				
	msg->AddInt32("read", static_cast<BIntegerField *>(row->GetField(COL_READ))->Value());				
	msg->AddString("title", static_cast<BStringField *>(row->GetField(COL_THREADNAME))->String());				
	msg->AddString("dat", static_cast<BStringField *>(row->GetField(COL_DATNAME))->String());				
	msg->AddString("host", fBoardUri);
	const char *dat = static_cast<BStringField *>(row->GetField(COL_DATNAME))->String();
	BString url;
	url << fBoardUri << dat;
	msg->AddString("url", url);
	return true;
}
////////////////////////////////////////////////////////////////////////////////
// ListInvoked
// 
//
////////////////////////////////////////////////////////////////////////////////
void
ThreadListView::ListInvoked ()
{
	BMessage openMessage(kMSG_OPEN_THREAD);
	this->CurrentQuery(&openMessage);
	DisplayThreadView* aView = static_cast<DisplayThreadView*>(this->Window()->FindView("Display Thread"));
	BMessenger(aView).SendMessage(&openMessage);
}
////////////////////////////////////////////////////////////////////////////////
// FindRowStringField
// 検索。
// Zeta 2スレ643のBe名無しさんより
////////////////////////////////////////////////////////////////////////////////
BRow*
ThreadListView::FindRowStringField(const char *label, int32 fieldNo)
{
	int32 count = this->CountRows();
	int32 index = this->IndexOf(this->CurrentSelection());
	
	//カレントセレクションからサーチ
	for (int32 i = index + 1; i < count; i++) {
	BStringField *dat = dynamic_cast<BStringField*>(RowAt(i)->GetField(fieldNo));
	if (dat && strstr(dat->String(), label) )
		return RowAt(i);
	}
	//カレントセレクションへ戻る。
	for (int32 i = 0; i <= index; i++) {
	BStringField *dat = dynamic_cast<BStringField*>(RowAt(i)->GetField(fieldNo));
	if (dat && strstr(dat->String(), label) )
		return RowAt(i);
	}
	return NULL;
}
////////////////////////////////////////////////////////////////////////////////
// DoFind
//
//
////////////////////////////////////////////////////////////////////////////////
void
ThreadListView::DoFind(BMessage *msg)
{
	BRow *row = FindRowStringField(msg->FindString("findthis"), COL_THREADNAME);
	if (row) {
		this->SetFocusRow(row, true);
		this->ScrollToRow(row);
	}else{
		beep();
	}
}
////////////////////////////////////////////////////////////////////////////////
// DoUpdate
//
//
////////////////////////////////////////////////////////////////////////////////
void
ThreadListView::DoUpdate(BMessage *msg)
{
	BRow *row = this->FindRowStringField(msg->FindString("dat"), COL_DATNAME);
	if (!row)	return;
	
	int32 thrdno = msg->FindInt32("thread_no");
	BIntegerField* field = dynamic_cast<BIntegerField*>(row->GetField(COL_READ));
	field->SetValue(thrdno);
	field = dynamic_cast<BIntegerField*>(row->GetField(COL_RES));
	if (field->Value() < thrdno) field->SetValue(thrdno);
	//20060701
	BBitmapField *bfield = dynamic_cast<BBitmapField*>(row->GetField(COL_STATUS));
	bfield->SetBitmap(GetPositinArrow(1, 1));
	//20060509
	this->Refresh();
}
////////////////////////////////////////////////////////////////////////////////
// LoadSubjectFile
//
//
////////////////////////////////////////////////////////////////////////////////
status_t
ThreadListView::LoadSubjectFile(BFile *file)
{
	return file->SetTo(fCacheDir, "subject.txt", B_READ_WRITE );
}
////////////////////////////////////////////////////////////////////////////////
// status_t ThreadListView::LoadSubectFromINet(BFile &file)
// ネットからsubject.txtを読み込む。
//
////////////////////////////////////////////////////////////////////////////////
status_t
ThreadListView::LoadSubectFromINet(BFile *file)
{
	if (static_cast<a2chViewer *>(be_app)->OffLineAlert() == B_ERROR)
		return B_ERROR;

	http_info hinfo;
	hinfo.gzipped = true;
	hinfo.body_length = 0;
	hinfo.modified = "";

	if(this->LoadSubjectFile(file) == B_OK){
		file->ReadAttrString("Last Modified", &hinfo.modified);
	}

	BMallocIO outbuf;
	char host[64], subdir[64], filename[64];
	uri2host(fBoardUri.String(), host, subdir);
	sprintf(filename,"%s/subject.txt", subdir);

	status_t sts;
	sts = get(host, filename, outbuf, hinfo);

	if(sts == HTTP_NOT_MODIFIED) return B_OK;
	if(sts != HTTP_OK) return B_ERROR; 
	
	fCacheDir->CreateFile("subject.txt", file);
	
	file->Write(outbuf.Buffer(), outbuf.BufferLength());
	file->WriteAttrString("Last Modified", &hinfo.modified);
	
	file->Seek(0, SEEK_SET);
	return B_OK;
}
////////////////////////////////////////////////////////////////////////////////
// LoadList
// スレ一覧リストを作成。datnameがある場合は指定されたスレも開く。
//
////////////////////////////////////////////////////////////////////////////////
void
ThreadListView::LoadList(bool uselocal, const char* datname)
{
BFile *file = new BFile;
status_t sts = B_OK;

	this->StopLoad();
	this->Clear();

	if(uselocal){
		if( this->LoadSubjectFile(file) != B_OK){
			if(static_cast<a2chViewer *>(be_app)->Alert() == B_OK)
				sts = this->LoadSubectFromINet(file);
			else{
				(new BAlert("","ファイルをロードできません", "OK"))->Go();
				return;
			}
		}
	}else{
		sts = this->LoadSubectFromINet(file);
	}
	//2005/05/03
	if(sts == B_ERROR){
		(new BAlert("","サーバーに接続できません", "OK"))->Go();
		return;
	}

	ThreadListView::Reader *reader = new ThreadListView::Reader(this, file, datname, fStopSem);
	fThread = spawn_thread(Reader::Run, "ListReader", B_NORMAL_PRIORITY, reader);
	resume_thread(fThread);

}
////////////////////////////////////////////////////////////////////////////////
// StopLoad
// 読込み用スレッドを停止。（MDRより）
//
////////////////////////////////////////////////////////////////////////////////
void
ThreadListView::StopLoad()
{
	Window()->Unlock();

	thread_info	info;
	if (fThread != 0 && get_thread_info(fThread, &info) == B_NO_ERROR) {
		fStopLoading = true;
		acquire_sem(fStopSem);
		int32 result;
		wait_for_thread(fThread, &result);
		fThread = 0;
		release_sem(fStopSem);
		fStopLoading = false;
	}

	Window()->Lock();
}
////////////////////////////////////////////////////////////////////////////////
// ThreadListView::SetArrow
//
//
////////////////////////////////////////////////////////////////////////////////
BBitmap *
ThreadListView::GetPositinArrow(int32 newposition, int32 oldposion)
{
	if(oldposion <= 0)
		return NULL;

	if(newposition < oldposion){
		return fBlueUpArrow;
	}else if(newposition == oldposion){
		return fYellowEqaulArrow;
	}else{
		return fRedDownArrow;
	}	
}
////////////////////////////////////////////////////////////////////////////////
// ThreadListView::Reader::Reader
// dat読込みスレッド
// 20050627:datnameをポインタでなくコピーで保持するように修正。
////////////////////////////////////////////////////////////////////////////////
ThreadListView::Reader::Reader(ThreadListView *view, BFile *datfile, const char * datname, sem_id sem)
:
fView(view),
fDatFile(datfile),
fOpenDatName(datname),
fStopSem(sem)
{
}
////////////////////////////////////////////////////////////////////////////////
// ThreadListView::Reader::makeThreadRow
//
//
////////////////////////////////////////////////////////////////////////////////
BRow*
ThreadListView::Reader::makeThreadRow(char *title, char *datname, int resnum, int threadposion)
{
BRow *row = new BRow();

	BEntry ent;
	BNode node; 		
	fView->fCacheDir->FindEntry(datname, &ent);
	node.SetTo(&ent);

	//スレ順
	row->SetField (new BIntegerField (threadposion), COL_NO);

	//スレの上下ステータス
	int32 oldthreadposion = 0;
	if (ent.Exists()){
		node.ReadAttr(DAT_ATTR_THREADNO, B_INT32_TYPE, 0, &oldthreadposion, sizeof(int32));
	}
	row->SetField (new BBitmapField (fView->GetPositinArrow(threadposion, oldthreadposion)), COL_STATUS);

	if (ent.Exists()){
		node.WriteAttr(DAT_ATTR_THREADNO, B_INT32_TYPE, 0, &threadposion, sizeof(int32));
	}

	//タイトル
	row->SetField (new BStringField (title), COL_THREADNAME);

	//レス数
	row->SetField (new BIntegerField (resnum), COL_RES);

	//既読
	int32 thread_read_no = 0;
	if (ent.Exists()){
		node.ReadAttr(DAT_ATTR_THREADCOUNT, B_INT32_TYPE, 0, &thread_read_no, sizeof(int32));
	}
	row->SetField (new BIntegerField (thread_read_no), COL_READ);

	//更新日付
	row->SetField (new BStringField (""), COL_DATE);

	//datファイル名
	row->SetField (new BStringField (datname), COL_DATNAME);

return row;
}
////////////////////////////////////////////////////////////////////////////////
// ThreadListView::Reader::Run
//20060510 lock見直し
//
////////////////////////////////////////////////////////////////////////////////
status_t
ThreadListView::Reader::Run(void *_this)
{
#define NAMEBUFSIZ 256
char datname[NAMEBUFSIZ], title[NAMEBUFSIZ*2];
int tn = 0;

	Reader *reader = static_cast<Reader *>(_this);
	BRow *invokedRow = NULL;
	
	if (!reader->Lock())
		return B_INTERRUPTED;

	ThreadListView *view = reader->fView;
	BFile *file = reader->fDatFile;

	BMallocIO utf8;
	cutf8(file, &utf8);
	delete file;
	
	const char *s = static_cast<const char *>(utf8.Buffer());
	const char *se = s + utf8.BufferLength();


	view->Window()->Lock();
	while(s < se){
		if(view->fStopLoading) goto done;

		char *p = datname;
		// get dat
		//20051209:machi bbs対応。
		while (*s != 't' && *s != 'i') *p++ = *s++;
		*p++ = *s;
		*p = 0;
		
		// skip delimter
		if (*(s + 1) == '<'){
			s += 3; //skip "t<>"
		}else{
			s += 2; //skip "t "
		}

		char *ss = title;
		while(*s != 10) *ss++ = *s++;
		s++; //skip 10
		*ss = '\0';
		while(*ss != ')') --ss;
		*ss = '\0';
		while(*ss != '(') --ss;
		int threadnum = atoi(ss + 1);
		*ss = '\0';
		tn++;

		BRow *row = reader->makeThreadRow(title, datname, threadnum, tn);

		view->AddRow (row);
		if (!invokedRow && reader->fOpenDatName.Length()) {
			if(!reader->fOpenDatName.Compare(datname)){
				invokedRow = row;
				view->SetFocusRow(invokedRow, true);
				view->ListInvoked();
			}
		}
	}

	//スレが指定されてない場合は一覧先頭にスクロール。
	if (!reader->fOpenDatName.Length()) {
		view->ScrollToRow(view->RowAt(static_cast<int32>(0)));
	}else{
		if(!invokedRow){
			beep();
			(new BAlert("","指定スレがありません。","Oh!"))->Go();
		}else{
			//指定スレにスクロール
			view->ScrollToRow(invokedRow);
		}
	}
	
done:
	view->Window()->Unlock();

//done:
	reader->Unlock();
	delete reader;
	return B_NO_ERROR;
}

////////////////////////////////////////////////////////////////////////////////
// ThreadListView::Reader::Unlock
//
//
////////////////////////////////////////////////////////////////////////////////
status_t
ThreadListView::Reader::Unlock()
{
	return release_sem(fStopSem);
}
////////////////////////////////////////////////////////////////////////////////
// ThreadListView::Reader::Lock
//
//
////////////////////////////////////////////////////////////////////////////////
bool
ThreadListView::Reader::Lock()
{
	if (acquire_sem_etc(fStopSem, 1, B_TIMEOUT, 0) != B_NO_ERROR)
		return false;
	return true;
}

