//------------------------------------------------------------------------------ 
//      Copyright (c) 2004, Be Nanashi-San 
// 
//      Permission is hereby granted, free of charge, to any person obtaining a 
//      copy of this software and associated documentation files (the "Software"), 
//      to deal in the Software without restriction, including without limitation 
//      the rights to use, copy, modify, merge, publish, distribute, sublicense, 
//      and/or sell copies of the Software, and to permit persons to whom the 
//      Software is furnished to do so, subject to the following conditions: 
// 
//      The above copyright notice and this permission notice shall be included in 
//      all copies or substantial portions of the Software. 
// 
//      THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
//      IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
//      FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
//      AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//      LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//      FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
//      DEALINGS IN THE SOFTWARE. 
// 
//      File Name:   mainWindow.cpp
//      Author:      Be Nanashi-San 
//      Description: 	 
//------------------------------------------------------------------------------
#include <Beep.h>
#include <Debug.h> // for PRINT()
#include <FindDirectory.h>
#include <Menu.h>
#include <MenuBar.h>
#include <MenuItem.h>
#include <Message.h>
#include <Messenger.h>
#include <Path.h>
#include <ScrollView.h>
#include <String.h>
#include <Window.h>
//#include <malloc.h>
#include <stdio.h>
#include <string.h>

#include "2chPositive.h"
#include "BoardListView.h"
#include "BoardOutLineListView.h"
#include "Bookmark2ch.h"
#include "DisplayThreadView.h"
#include "SplitPane.h"
#include "StatusBarView.h"
#include "ThreadListView.h"
#include "Util.h"
#include "mainWindow.h"
#include "messages.h"
#include "setting.h"

////////////////////////////////////////////////////////////////////////////////
// mainWindow
//
//
////////////////////////////////////////////////////////////////////////////////
mainWindow::mainWindow(BRect frame)
				: BWindow(frame, "2chPositive", B_DOCUMENT_WINDOW_LOOK, B_NORMAL_WINDOW_FEEL, B_ASYNCHRONOUS_CONTROLS )
{
	BMenuBar *menubar = new BMenuBar(this->Bounds(), "");
	this->AddChild(menubar);

	BRect contentRect(this->Bounds());
	contentRect.top = menubar->Bounds().Height() + 1.0;
	contentRect.bottom -= B_H_SCROLL_BAR_HEIGHT;

	BView * contentView = new BView(contentRect, "ContentView", B_FOLLOW_ALL_SIDES, 0);
	contentView->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	this->AddChild(contentView);
	
	BRect statusBarRect = this->Bounds();
	statusBarRect.top = statusBarRect.bottom - (B_H_SCROLL_BAR_HEIGHT - 1);
	statusBarRect.right -= B_V_SCROLL_BAR_WIDTH;
	fStatusBar = new StatusBarView(statusBarRect);
	this->AddChild(fStatusBar);

	fThreadListView = new ThreadListView(contentView->Bounds());
	fDisplayThreadView = new DisplayThreadView(contentView->Bounds());

	fHSplitPane = new SplitPane(contentView->Bounds(), fThreadListView, fDisplayThreadView, B_FOLLOW_ALL_SIDES);
	fHSplitPane->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
   	fHSplitPane->SetAlignment(B_HORIZONTAL);
   	fHSplitPane->SetBarAlignmentLocked(true);
	fHSplitPane->SetBarPosition(theSettings->GetPoint(kSETTING_H_SPLITTER_POINT));
	
	fBoardListView = new BoardListView(contentView->Bounds());

	fVSplitPane = new SplitPane(contentView->Bounds(), fBoardListView, fHSplitPane, B_FOLLOW_ALL_SIDES);
	fVSplitPane->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
   	fVSplitPane->SetAlignment(B_VERTICAL);
   	fVSplitPane->SetBarAlignmentLocked(true);

	fVSplitPane->SetBarPosition(theSettings->GetPoint(kSETTING_V_SPLITTER_POINT));
	contentView->AddChild(fVSplitPane);
	
	this->SetupMenu(menubar);
	(fBoardListView->OutLineView())->MakeFocus(true);
}
////////////////////////////////////////////////////////////////////////////////
// QuitRequested
//
//
////////////////////////////////////////////////////////////////////////////////
bool
mainWindow::QuitRequested()
{
	theSettings->SetRect(kSETTING_WINDOW_RECT,this->Frame());
	theSettings->SetPoint(kSETTING_H_SPLITTER_POINT,fHSplitPane->GetBarPosition());
	theSettings->SetPoint(kSETTING_V_SPLITTER_POINT,fVSplitPane->GetBarPosition());
	be_app_messenger.SendMessage(B_QUIT_REQUESTED);
	return true;
}
////////////////////////////////////////////////////////////////////////////////
// DispatchMessage
//
//
////////////////////////////////////////////////////////////////////////////////
void
mainWindow::DispatchMessage(BMessage *msg, BHandler *handler)
{
	if (msg->what == B_SIMPLE_DATA && msg->HasRef("refs")) {
		BMessage message(*msg);
		message.what = B_REFS_RECEIVED;
		be_app_messenger.SendMessage(&message);
	}else{
		BWindow::DispatchMessage(msg, handler);
	}
}
////////////////////////////////////////////////////////////////////////////////
// doOpenBookmark
//
//
////////////////////////////////////////////////////////////////////////////////
void
mainWindow::doOpenBookmark()
{
	BPath path;
	find_directory(B_USER_SETTINGS_DIRECTORY, &path);
	path.Append(kBOOKMARK_DIR_NAME);
	get_ref_for_path(path.Path(), &fBookmarkRef);

	BMessage message(B_REFS_RECEIVED);
	message.AddRef("refs", &fBookmarkRef);

	BMessenger messenger("application/x-vnd.Be-TRAK");
	messenger.SendMessage(&message);

}
////////////////////////////////////////////////////////////////////////////////
// MessageReceived
//
//
////////////////////////////////////////////////////////////////////////////////
void
mainWindow::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
		case B_SIMPLE_DATA:{
			PRINT(("mainWindow "));
		//	PRINT_OBJECT((&msg));
			break;
		}			
		//zeta 2 スレ@713のBe名無しさんより。
		case kMENU_BOOKMARK_ADD:{
			BMessage query('qury');
			if(this->CurrentQuery(&query)){
				fBookmark->AddBookmark(&query);
			}
			break;
		}
		case kMENU_BOOKMARK_SHOW:
			this->doOpenBookmark();
			break;

		case kMSG_BOOKMARK:
			(fBoardListView->OutLineView())->OpenBookmark(msg);
			break;

		case B_NODE_MONITOR:
			fBookmark->WatchMessage(msg);
			break;

		case B_CUT:
		case B_COPY:
		case B_PASTE:
		case B_SELECT_ALL:
 			BMessenger(this->CurrentFocus()).SendMessage(msg);
			break;

		case kMSG_FIND:{
			int32 i;
			msg->FindInt32("targetview", &i);
			BHandler *handler[3]={fDisplayThreadView,fThreadListView,fBoardListView};
			BHandler *h = handler[i];	// 20050702:gcc2 fix
			BMessenger(h).SendMessage(msg);
			}
			break;

 //		case kMENU_THREAD_REFREASH:
 		case kMSG_JUMP:
 		case kMEG_WRITE_BUTTON: // StatusBarより
			BMessenger(fDisplayThreadView).SendMessage(msg);
			break;
		
		case kMSG_UPDATE_THREADCOUNT:
			BMessenger(fThreadListView).SendMessage(msg);
			break;
		
		case kMENU_FILE_OFFLINE:
		case kMSG_STATUS_MESSAGE:
			BMessenger(fStatusBar).SendMessage(msg);
			break;
		
		case kMENU_EDIT_OPENPREFERNCE:
			be_app_messenger.SendMessage(kMENU_EDIT_OPENPREFERNCE);
			break;
		
		case kMENU_FILE_NEWWINDOW:
		case kMENU_HELP:
			be_app_messenger.SendMessage(kNOT_INPLEMENTED);
			break;

		default:
			BWindow::MessageReceived (msg);
			break;
	}
}
////////////////////////////////////////////////////////////////////////////////
// SetupMenu
// メニュー作成。
//
////////////////////////////////////////////////////////////////////////////////
void
mainWindow::SetupMenu(BMenuBar *menubar)
{
BMenuItem *item;
	// File メニュー
	BMenu *menu = new BMenu("ファイル");
	menu->AddItem(MakeItem(this, "新規ウインドウ", kMENU_FILE_NEWWINDOW, 'N'));
	menu->AddSeparatorItem();
	menu->AddItem(MakeItem(this, "閉じる", B_QUIT_REQUESTED, 'W'));
	menu->AddSeparatorItem();
	item = MakeItem(this, "オフライン", kMENU_FILE_OFFLINE);
	item->SetMarked(theSettings->GetBool(kSETTING_OFFLINE));
	menu->AddItem(item);
	menu->AddSeparatorItem();
	menu->AddItem(MakeItem(be_app, "2chPositiveについて" B_UTF8_ELLIPSIS, B_ABOUT_REQUESTED));
	menu->AddSeparatorItem();
	menu->AddItem(MakeItem(this, "終了", B_QUIT_REQUESTED, 'Q'));
	menubar->AddItem(menu);
	
	// Edit メニュー	
	menu = new BMenu("編集");
	menu->AddItem(MakeItem(this, "切り取り" , B_CUT, 'X'));
	menu->AddItem(MakeItem(this, "コピー" , B_COPY, 'C'));
	menu->AddItem(MakeItem(this, "貼り付け" , B_PASTE, 'V'));
	menu->AddSeparatorItem();
	menu->AddItem(MakeItem(this, "全てを選択" , B_SELECT_ALL, 'A'));
	menu->AddSeparatorItem();
	menu->AddItem(MakeItem(be_app, "検索" , kMENU_EDIT_FIND, 'F'));
	menu->AddItem(MakeItem(be_app, "次検索" , kMENU_EDIT_FINDAGAIN, 'G'));
	menu->AddSeparatorItem();
	menu->AddItem(MakeItem(this, "設定" B_UTF8_ELLIPSIS, kMENU_EDIT_OPENPREFERNCE));
	fEditMenu = menu;
	menubar->AddItem(menu);

	// BoardListメニュー	
	menu = new BMenu("板一覧");
	menu->AddItem(MakeItem(fBoardListView, "再読込", kMENU_BOARDLIST_REFREASH));
	menu->AddSeparatorItem();
	menu->AddItem(MakeItem(fBoardListView, "全てを展開", kMENU_BOARDLIST_EXPANDALL, '+'));
	menu->AddItem(MakeItem(fBoardListView, "全てを省略", kMENU_BOARDLIST_COLLAPSEALL, '-'));
	menubar->AddItem(menu);
	fBoardMenu = menu;
	
	// ThreadListメニュー	
	menu = new BMenu("スレ一覧");
	menu->AddItem(MakeItem(fThreadListView, "再読込", kMENU_THREADLIST_REFREASH, 'R', B_COMMAND_KEY | B_SHIFT_KEY));
	fThreadMenu = menu;
	menubar->AddItem(menu);
	
	// Threadメニュー	
	menu = new BMenu("スレ");
	menu->AddItem(MakeItem(fDisplayThreadView, "再読込", kMENU_THREAD_REFREASH, 'R'));
	menu->AddSeparatorItem();
	menu->AddItem(MakeItem(this, "書き込み", kMEG_WRITE_BUTTON));
	menu->AddItem(MakeItem(be_app, "スレ立て", kNOT_INPLEMENTED));
	menu->AddSeparatorItem();
	menu->AddItem(MakeItem(be_app, "ジャンプ", kMENU_THREAD_JUMP, ','));
	menu->AddItem(MakeItem(be_app, "webブラウザで開く", kNOT_INPLEMENTED));
	menu->AddSeparatorItem();
	menu->AddItem(MakeItem(fDisplayThreadView, "最初から100スレ表示。", kMENU_THREAD_GO_TOP100, 'S', B_COMMAND_KEY | B_SHIFT_KEY));
	menu->AddItem(MakeItem(fDisplayThreadView, "最新100スレ表示。", kMENU_THREAD_GO_LAST100, 'E', B_COMMAND_KEY | B_SHIFT_KEY));
	menu->AddItem(MakeItem(fDisplayThreadView, "前100を表示", kMENU_THREAD_GO_PREV100, 'P', B_COMMAND_KEY | B_SHIFT_KEY));
	menu->AddItem(MakeItem(fDisplayThreadView, "次100スレ表示。", kMENU_THREAD_GO_NEXT100, 'N', B_COMMAND_KEY | B_SHIFT_KEY));
	fThreadMenu = menu;
	menubar->AddItem(menu);
	
	// Bookmarks メニュー	
	menu = new BMenu("お気に入り");
	menu->AddItem(MakeItem(this, "お気に入りに追加", kMENU_BOOKMARK_ADD));
	menu->AddItem(MakeItem(this, "お気に入りを表示", kMENU_BOOKMARK_SHOW));
	menu->AddSeparatorItem();
	fBookmarkMenu = menu;
	fBookmark = new Bookmark2ch(menu, BMessenger(this));
	menubar->AddItem(menu);

	// Helpメニュー	
	menu = new BMenu("ヘルプ");
	menu->AddItem(MakeItem(this, "ヘルプ", kMENU_HELP));
	menubar->AddItem(menu);
}
////////////////////////////////////////////////////////////////////////////////
// MenusBeginning
// CurrentFocusによってAddBookmarkが可能か調べる
// zeta 2 スレ@757のBe名無しさんより。
////////////////////////////////////////////////////////////////////////////////
void
mainWindow::MenusBeginning()
{
	BView *cView = CurrentFocus();
// Edit
	bool b = (cView ==  fDisplayThreadView->FindView("2ch:TextView")); 
	(fEditMenu->ItemAt(0))->SetEnabled(b);
	(fEditMenu->ItemAt(1))->SetEnabled(b);
	(fEditMenu->ItemAt(2))->SetEnabled(b);
	(fEditMenu->ItemAt(4))->SetEnabled(b);
// Thread List

// Thread

// bookmark
//20050626:CurrentQueryの値を使うように修正。
	(fBookmarkMenu->ItemAt(0))->SetEnabled(this->CurrentQuery(NULL));
}
////////////////////////////////////////////////////////////////////////////////
// CurrentQuery
// CurrentFocusによってCurrentQueryを切り分ける
// zeta 2 スレ@757のBe名無しさんより。
////////////////////////////////////////////////////////////////////////////////
// TODO(Refactoring): 各viewの親クラスに入らないか？
bool
mainWindow::CurrentQuery(BMessage *query)
{
	BView *cView = CurrentFocus();
	if (!cView) return false;
	
	if (cView == fBoardListView->OutLineView()){
		return (fBoardListView->OutLineView())->CurrentQuery(query);
	}
	
	if (cView == fThreadListView) {
		return fThreadListView->CurrentQuery(query);
	}
	return false;
}
